#!/usr/bin/perl
#
#  '$RCSfile$'
#  Copyright: 2000 Regents of the University of California 
#
#   '$Author: sgarg $'
#     '$Date: 2004-12-16 14:33:10 -0800 (Thu, 16 Dec 2004) $'
# '$Revision: 2334 $' 
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

#
# This is a CGI application for inserting metadata documents into
# the Metacat database.  It utilizes the Metacat.pm module for most work.
# In this script, we process the form fields passed in from a POST, insert a
# metadata document and an ACL document.

use Metacat;
use AppConfig qw(:expand :argcount);
use XML::LibXML;
use XML::LibXSLT;
use Template;
use Net::SMTP;
use CGI qw/:standard :html3/;
use strict;

# Global configuration paramters
#my $cfgdir = "@install-dir@";
#my $cfgdir = "/usr/local/devtools/tomcat/webapps/knb/style/skins";
my $cfgdir = "@install-dir@@style-skins-relpath@";
my $tmpdir = "@temp-dir@";
my $templatesdir = "@install-dir@@style-common-relpath@/templates";
my $now = time;
my $xslConvDir = "$cfgdir/lib/style/";

# Import all of the HTML form fields as variables
import_names('FORM');

# Set up the hash for returning data to the HTML templates
my $templateVars = { 'status' => 'success' };
my $error = 0;
my @errorMessages;

# create a new AppConfig object and load our config parameters
# note that this requires the form submission to have a "cfg" paramter
# to determine which config file to load
my $config = AppConfig->new({
    GLOBAL => { ARGCOUNT => ARGCOUNT_ONE, } });

$config->define("metacatUrl");
$config->define("username");
$config->define("password");
$config->define("ldapUrl");
$config->define("defaultScope");
$config->define("organization");
$config->define("orgabbrev");
$config->define("orgurl");
$config->define("accesspubid");
$config->define("accesssysid");
$config->define("datasetpubid");
$config->define("datasetsysid");
$config->define("showSiteList", { DEFAULT => 'true'} );
$config->define("lsite", { DEFAULT => 'station'} );
$config->define("usite", { DEFAULT => 'Station'} );
$config->define("showWgList", { DEFAULT => 'true'} );
$config->define("showOrganization", { DEFAULT => 'true'} );
$config->define("hasKeyword", { DEFAULT => 'true'} );
$config->define("hasTemporal", { DEFAULT => 'true'} );
$config->define("hasSpatial", { DEFAULT => 'true'} );
$config->define("hasTaxonomic", { DEFAULT => 'true'} );
$config->define("hasMethod", { DEFAULT => 'true'} );
$config->define("temporalRequired", { DEFAULT => 'true'} );
$config->define("spatialRequired", { DEFAULT => 'true'} );
$config->define("mailhost");
$config->define("sender");
$config->define("recipient");
$config->define("adminname");
if ($FORM::cfg eq 'nceas') {
    $config->define("nceas_db");
    $config->define("nceas_db_user");
    $config->define("nceas_db_password");
}
$config->define("responseTemplate", { DEFAULT => 'crap.tmpl'} );
$config->define("entryFormTemplate", { DEFAULT => 'crap.tmpl'} );
$config->define("guideTemplate", { DEFAULT => 'crap.tmpl'} );
$config->define("confirmDataTemplate", { DEFAULT => 'crap.tmpl'} );
$config->define("deleteDataTemplate", { DEFAULT => 'crap.tmpl'} );
$config->define("debug", { DEFAULT => '0'} );
$config->define("lat", { ARGCOUNT => ARGCOUNT_HASH} );
$config->define("lon", { ARGCOUNT => ARGCOUNT_HASH} );

if (! hasContent($FORM::cfg)) {
    $error = "Application misconfigured.  Please contact the administrator.";
    push(@errorMessages, $error);
} else {
    my $cfgfile = $cfgdir . "/" . $FORM::cfg . "/" . $FORM::cfg . ".cfg";
    $config->file($cfgfile);
}

my $metacatUrl = $config->metacatUrl();
my $username = $config->username();
my $password = $config->password();
my $ldapUrl = $config->ldapUrl();
my $defaultScope = $config->defaultScope();
my $organization = $config->organization();
my $orgabbrev = $config->orgabbrev();
my $orgurl = $config->orgurl();
my $orgfilter = $organization;
      $orgfilter =~ s/ /%20/g;
my $responseTemplate = $config->responseTemplate();
my $entryFormTemplate = $config->entryFormTemplate();
my $deleteDataTemplate = $config->deleteDataTemplate();
my $guideTemplate = $config->guideTemplate();
my $confirmDataTemplate = $config->confirmDataTemplate();
my $accesspubid = $config->accesspubid();
my $accesssysid = $config->accesssysid();
my $datasetpubid = $config->datasetpubid();
my $datasetsysid = $config->datasetsysid();
my $showSiteList = $config->showSiteList();
my $lsite = $config->lsite();
my $usite = $config->usite();
my $showWgList = $config->showWgList();
my $showOrganization = $config->showOrganization();
my $hasKeyword = $config->hasKeyword();
my $hasTemporal = $config->hasTemporal();
my $hasSpatial = $config->hasSpatial();
my $hasTaxonomic = $config->hasTaxonomic();
my $hasMethod = $config->hasMethod();
my $temporalRequired = $config->temporalRequired();
my $spatialRequired = $config->spatialRequired();
my $mailhost = $config->mailhost();
my $sender = $config->sender();
my $recipient = $config->recipient();
my $adminname = $config->adminname();
my $nceas_db;
my $nceas_db_user;
my $nceas_db_password;
if ($FORM::cfg eq 'nceas') {
    $nceas_db = $config->nceas_db();
    $nceas_db_user = $config->nceas_db_user();
    $nceas_db_password = $config->nceas_db_password();
}
my $debug = $config->debug();
my $lat = $config->get('lat');
my $lon = $config->get('lon');

# Convert the lat and lon configs into usable data structures
my @sitelist;
my %siteLatDMS;
my %siteLongDMS;
foreach my $newsite (keys %$lat) {
    my ($latd, $latm, $lats, $latdir) = split(':', $lat->{$newsite});
    my ($lond, $lonm, $lons, $londir) = split(':', $lon->{$newsite});
    push(@sitelist, $newsite);
    $siteLatDMS{$newsite} = [ $latd, $latm, $lats, $latdir ];
    $siteLongDMS{$newsite} = [ $lond, $lonm, $lons, $londir ];
}

# set some configuration options for the template object
my $ttConfig = {
             INCLUDE_PATH => $templatesdir, 
             INTERPOLATE  => 0,                    
             POST_CHOMP   => 1,                   
             };

# create an instance of the template processor
my $template = Template->new($ttConfig) || die $Template::ERROR, "\n";

print "Content-type: text/html\n\n";

# Set up the template information that is common to all forms
$$templateVars{'cfg'} = $FORM::cfg;
$$templateVars{'recipient'} = $recipient;
$$templateVars{'adminname'} = $adminname;
$$templateVars{'organization'} = $organization;
$$templateVars{'orgabbrev'} = $orgabbrev;
$$templateVars{'orgurl'} = $orgurl;
$$templateVars{'orgfilter'} = $orgfilter;

debug("Registry: Initialized");
# Process the form based on stage parameter. 
if ($FORM::stage =~ "guide") {
    # Send back the information on how to fill the form
    $$templateVars{'section'} = "Guide on How to Complete Registry Entries";
    $template->process( $guideTemplate, $templateVars);
    exit(0);

} elsif ($FORM::stage =~ "insert") {
    # The user has entered the data. Do data validation and send back data 
    # to confirm the data that has been entered. 
    toConfirmData();
    exit(0);

}elsif ($FORM::dataWrong =~ "No, go back to editing" && $FORM::stage =~ "confirmed") {
    # The user wants to correct the data that he has entered. 
    # Hence show the data again in entryData form. 
    confirmDataToReEntryData();
    exit(0);

}elsif ($FORM::stage =~ "modify") {
    # Modification of a file has been requested. 
    # Show the form will all the values filled in.
    my @sortedSites;
    foreach my $site (sort @sitelist) {
        push(@sortedSites, $site);
    }
    $$templateVars{'siteList'} = \@sortedSites;
    $$templateVars{'section'} = "Modification Form";
    $$templateVars{'docid'} = $FORM::docid;
    modifyData();
    exit(0);

}elsif ($FORM::stage =~ "delete_confirm") {

    # Result from deleteData form. 
    if($FORM::deleteData =~ "Delete data"){
    # delete Data
    deleteData(1);    
    exit(0);
    } else {
    # go back to search page. 
    exit(0);
    }

}elsif ($FORM::stage =~ "delete") {
    # Deletion of a file has been requested. 
    # Ask for username and password using deleteDataForm
    $$templateVars{'docid'} = $FORM::docid;
    $template->process( $deleteDataTemplate, $templateVars);
    exit(0);

}elsif ($FORM::stage !~ "confirmed") {
    # None of the stages have been reached and data is not being confirmed. 
    # Hence, send back entry form for entry of data.  
    debug("Registry: Sending form");
    my @sortedSites;
    foreach my $site (sort @sitelist) {
        push(@sortedSites, $site);
    }
    
    if ($FORM::cfg eq 'nceas') {
        my $projects = getProjectList();
        $$templateVars{'projects'} = $projects;
        $$templateVars{'wg'} = \@FORM::wg;
    }

    $$templateVars{'showSiteList'} = $showSiteList;
    $$templateVars{'lsite'} = $lsite;
    $$templateVars{'usite'} = $usite;
    $$templateVars{'showWgList'} = $showWgList;
    $$templateVars{'showOrganization'} = $showOrganization;
    $$templateVars{'hasKeyword'} = $hasKeyword;
    $$templateVars{'hasTemporal'} = $hasTemporal;
    $$templateVars{'hasSpatial'} = $hasSpatial;
    $$templateVars{'hasTaxonomic'} = $hasTaxonomic;
    $$templateVars{'hasMethod'} = $hasMethod;
    $$templateVars{'temporalRequired'} = $temporalRequired;
    $$templateVars{'spatialRequired'} = $spatialRequired;

    $$templateVars{'siteList'} = \@sortedSites;
    $$templateVars{'section'} = "Entry Form";
    $$templateVars{'docid'} = "";
    debug("Registry: Sending form: ready to process template");
    $template->process( $entryFormTemplate, $templateVars);
    debug("Registry: Sending form: template processed");
    exit(0);
}

# Confirm stage has been reached. Enter the data into metacat. 

# Initialize some global vars
my $latDeg1 = "";
my $latMin1 = "";
my $latSec1 = "";
my $hemisphLat1 = "";
my $longDeg1 = "";
my $longMin1 = "";
my $longSec1 = "";
my $hemisphLong1 = "";
my $latDeg2 = "";
my $latMin2 = "";
my $latSec2 = "";
my $hemisphLat2 = "";
my $longDeg2 = "";
my $longMin2 = "";
my $longSec2 = "";
my $hemisphLong2 = "";

# validate the input form parameters
my $invalidParams;

if (! $error) {
    $invalidParams = validateParameters(1);
    if (scalar(@$invalidParams)) {
        $$templateVars{'status'} = 'failure';
        $$templateVars{'invalidParams'} = $invalidParams;
        $error = 1;
    }
}


my $metacat;
my $docid;
if (! $error) {
    # Parameters have been validated and Create the XML document

    my $xmldoc = createXMLDocument();

    # Write out the XML file for debugging purposes
    #my $testFile = $tmpdir . "/test.xml";

    # Create a  metacat object
    $metacat = Metacat->new();
    if ($metacat) {
        $metacat->set_options( metacatUrl => $metacatUrl );
    } else {
        #die "failed during metacat creation\n";
        push(@errorMessages, "Failed during metacat creation.");
    }

    # Login to metacat
    my $userDN = $FORM::username;
    my $userOrg = $FORM::organization;
    my $userPass = $FORM::password;
    my $dname = "uid=$userDN,o=$userOrg,dc=ecoinformatics,dc=org";
    
    my $xmldocWithDocID = $xmldoc;
    
    my $errorMessage = "";
    my $response = $metacat->login($dname, $userPass);
    if (! $response) {
        push(@errorMessages, $metacat->getMessage());
        push(@errorMessages, "Failed during login.\n");
    }

    debug( "Registry: A");
    if ($FORM::docid eq "") {
        debug( "Registry: B1");
        # document is being inserted 
        my $notunique = "NOT_UNIQUE";
        while ($notunique eq "NOT_UNIQUE") {
            $docid = newAccessionNumber($defaultScope);
            
	    $xmldocWithDocID = $xmldoc;
            $xmldocWithDocID =~ s/docid/$docid/;

	    # Code for testing the xml file being inserted####
            #my $testFile = "/tmp/test.xml";
            #open (TFILE,">$testFile") || die ("Cant open xml file...\n");
            #print TFILE $xmldoc;
            #close(TFILE);
	    ####
    
            $notunique = insertMetadata($xmldocWithDocID, $docid);
            #  if (!$notunique) {
            # Write out the XML file for debugging purposes
            #my $testFile = $tmpdir . "/test-new.xml";
            #open (TFILE,">$testFile") || die ("Cant open xml file...\n");
            #print TFILE $newdoc;
            #close(TFILE);
            #   }

            # The id wasn't unique, so update our lastid file
            if ($notunique eq "NOT_UNIQUE") {
                debug( "Registry: Updating lastid (B1.1)");
                updateLastId($defaultScope);
            }
        }
        debug("Registry: B2");
        if ($notunique ne "SUCCESS") {
            debug("Registry: NO SUCCESS");
            debug("Message is: $notunique");
            push(@errorMessages, $notunique);
        }

        debug("Registry: B3");
    } else {
        # document is being modified
        $docid = $FORM::docid;
    
        my $x;
        my $y;
        my $z;

        ($x, $y, $z) = split(/\./, $docid); 
        $z++;
        $docid = "$x.$y.$z";
    
        $xmldoc =~ s/docid/$docid/;
        
        my $response = $metacat->update($docid, $xmldoc);

        if (! $response) {
            push(@errorMessages, $metacat->getMessage());
            push(@errorMessages, "Failed while updating.\n");  
        }

        if (scalar(@errorMessages)) {
            debug("Registry: ErrorMessages defined in modify.");

	    $$templateVars{'docid'} = $FORM::docid;
	    copyFormToTemplateVars();
            $$templateVars{'status'} = 'failure';
            $$templateVars{'errorMessages'} = \@errorMessages;
            $error = 1;
        } else {
	    $$templateVars{'docid'} = $docid;
	    $$templateVars{'cfg'} = $FORM::cfg;
	}

        #if (! $error) {
            #sendNotification($docid, $mailhost, $sender, $recipient);
        #}
    
        # Create our HTML response and send it back
        $$templateVars{'function'} = "modified";
        $$templateVars{'section'} = "Modification Status";
        $template->process( $responseTemplate, $templateVars);

        exit(0);
    }
}

debug("Registry: C");

if (scalar(@errorMessages)) {
    debug("Registry: ErrorMessages defined.");
    $$templateVars{'docid'} = $FORM::docid;
    copyFormToTemplateVars();
    $$templateVars{'status'} = 'failure';
    $$templateVars{'errorMessages'} = \@errorMessages;
    $error = 1;
} else {
    $$templateVars{'docid'} = $docid;
    $$templateVars{'cfg'} = $FORM::cfg;
}

#if (! $error) {
#sendNotification($docid, $mailhost, $sender, $recipient);
#}

# Create our HTML response and send it back
$$templateVars{'function'} = "submitted";
$$templateVars{'section'} = "Submission Status";

$template->process( $responseTemplate, $templateVars);

exit(0);


################################################################################
#
# Subroutine for updating a metacat id for a given scope to the highest value
#
################################################################################
sub updateLastId {
  my $scope = shift;

  my $errormsg = 0;
  my $docid = $metacat->getLastId($scope);

  if ($docid =~ /null/) {
      # No docids with this scope present, so do nothing
  } elsif ($docid) {
      # Update the lastid file for this scope
      (my $foundScope, my $id, my $rev) = split(/\./, $docid);
      debug("Docid is: $docid\n");
      debug("Lastid is: $id");
      my $scopeFile = $cfgdir . "/" . $FORM::cfg . "/" . $scope . ".lastid";
      open(LASTID, ">$scopeFile") or 
          die "Failed to open lastid file for writing!";
      print LASTID $id, "\n";
      close(LASTID);
  } else {
    $errormsg = $metacat->getMessage();
    debug("Error in getLastId: $errormsg");
  }
}

################################################################################
#
# Subroutine for inserting a document to metacat
#
################################################################################
sub insertMetadata {
  my $xmldoc = shift;
  my $docid = shift;

  my $notunique = "SUCCESS";
  debug("Registry: Starting insert (D1)");
  my $response = $metacat->insert($docid, $xmldoc);
  if (! $response) {
    debug("Registry: Response gotten (D2)");
    my $errormsg = $metacat->getMessage();
    debug("Registry: Error is (D3): ".$errormsg);
    if ($errormsg =~ /is already in use/) {
      $notunique = "NOT_UNIQUE";
      #print "Accession number already used: $docid\n";
    } elsif ($errormsg =~ /<login>/) {
      $notunique = "SUCCESS";
    } else {
      #print "<p>Dumping error on failure...</p>\n";
      #print "<p>", $errormsg, "</p>\n";
      #die "Failed during insert\n";
      #print "<p>Failed during insert</p>\n";
      $notunique = $errormsg;
    }
  }
  debug("Registry: Ending insert (D4)");

  return $notunique;
}

################################################################################
#
# Subroutine for generating a new accession number
#  Note: this is not threadsafe, assumes only one running process at a time
#  Also: need to check metacat for max id # used in this scope already
################################################################################
sub newAccessionNumber {
  my $scope = shift;
    
  my $docrev = 1;
  my $lastid = 1;

  my $scopeFile = $cfgdir . "/" . $FORM::cfg . "/" . $scope . ".lastid";
  if (-e $scopeFile) {
    open(LASTID, "<$scopeFile") or die "Failed to generate accession number!";
    $lastid = <LASTID>;
    chomp($lastid);
    $lastid++;
    close(LASTID);
  }
  open(LASTID, ">$scopeFile") or die "Failed to open lastid file for writing!";
  print LASTID $lastid, "\n";
  close(LASTID);

  my $docroot = "$scope.$lastid.";
  my $docid = $docroot . $docrev;
  return $docid;
}

################################################################################
# 
# Validate the parameters to make sure that required params are provided
#
################################################################################
sub validateParameters {
    my $chkUser = shift;
    my @invalidParams;

    push(@invalidParams, "Provider's first name is missing.")
        unless hasContent($FORM::providerGivenName);
    push(@invalidParams, "Provider's last name is missing.")
        unless hasContent($FORM::providerSurName);
    push(@invalidParams, "Name of site is missing.")
        unless (hasContent($FORM::site) || $FORM::site =~ /elect/ ||
                $FORM::cfg eq "nceas");
    push(@invalidParams, "Data set title is missing.")
        unless hasContent($FORM::title);
    push(@invalidParams, "Originator's first name is missing.")
        unless hasContent($FORM::origNamefirst0);
    push(@invalidParams, "Originator's last name is missing.")
        unless hasContent($FORM::origNamelast0);
    push(@invalidParams, "Abstract is missing.")
        unless hasContent($FORM::abstract);
    if($FORM::hasTemporal eq 'true'){
	push(@invalidParams, "Beginning year of data set is missing.")
	    unless (hasContent($FORM::beginningYear) || $FORM::temporalRequired ne 'true');
	push(@invalidParams, "Ending year has been specified but beginning year of data set is missing.")
	    if ((!hasContent($FORM::beginningYear)) && hasContent($FORM::endingYear));
    }
    push(@invalidParams, "Geographic description is missing.")
        unless (hasContent($FORM::geogdesc) || $FORM::spatialRequired ne 'true');

    if($FORM::beginningMonth eq "00"){
	if (hasContent($FORM::beginningYear)){
	    $FORM::beginningMonth = "01";
	} else {
	    $FORM::beginningMonth = "";
	}
    }
    if($FORM::beginningDay eq "00"){
	if (hasContent($FORM::beginningYear)){
	    $FORM::beginningDay = "01";
	} else {
	    $FORM::beginningDay = "";
	}
    }
    if($FORM::endingMonth eq "00"){
	if (hasContent($FORM::endingYear)){
	    $FORM::endingMonth = "01";
	} else {
	    $FORM::endingMonth = "";
	}
    }    
    if($FORM::endingDay eq "00"){
	if (hasContent($FORM::endingYear)){
	    $FORM::endingDay = "01";
	} else {
	    $FORM::endingDay = "";
	}
    }

    if (hasContent($FORM::beginningYear) && !($FORM::beginningYear =~ /[0-9][0-9][0-9][0-9]/)){
	push(@invalidParams, "Invalid beginning year specified.")
    }

    if (hasContent($FORM::endingYear) && !($FORM::endingYear =~ /[0-9][0-9][0-9][0-9]/)){
	push(@invalidParams, "Invalid ending year specified.")
    }
    
    # If the "use site" coord. box is checked and if the site is in 
    # the longitude hash ...  && ($siteLatDMS{$FORM::site})
    
    if($FORM::hasSpatial eq 'true'){
	if (($FORM::useSiteCoord) && ($siteLatDMS{$FORM::site}) ) {
        
	    $latDeg1 = $siteLatDMS{$FORM::site}[0];
	    $latMin1 = $siteLatDMS{$FORM::site}[1];
	    $latSec1 = $siteLatDMS{$FORM::site}[2];
	    $hemisphLat1 = $siteLatDMS{$FORM::site}[3];
	    $longDeg1 = $siteLongDMS{$FORM::site}[0];
	    $longMin1 = $siteLongDMS{$FORM::site}[1];
	    $longSec1 = $siteLongDMS{$FORM::site}[2];
	    $hemisphLong1 = $siteLongDMS{$FORM::site}[3];
	    
	}  else {
	    
	    $latDeg1 = $FORM::latDeg1;
	    $latMin1 = $FORM::latMin1;
	    $latSec1 = $FORM::latSec1;
	    $hemisphLat1 = $FORM::hemisphLat1;
	    $longDeg1 = $FORM::longDeg1;
	    $longMin1 = $FORM::longMin1;
	    $longSec1 = $FORM::longSec1;
	    $hemisphLong1 = $FORM::hemisphLong1;
	}

	if($latDeg1 > 90 || $latDeg1 < 0){
	    push(@invalidParams, "Invalid first latitude degrees specified.");
	}
	if($latMin1 > 59 || $latMin1 < 0){
	    push(@invalidParams, "Invalid first latitude minutes specified.");
	}
	if($latSec1 > 59 || $latSec1 < 0){
	    push(@invalidParams, "Invalid first latitude seconds specified.");
	}
	if($longDeg1 > 180 || $longDeg1 < 0){
	    push(@invalidParams, "Invalid first longitude degrees specified.");
	}
	if($longMin1 > 59 || $longMin1 < 0){
	    push(@invalidParams, "Invalid first longitude minutes specified.");
	}
	if($longSec1 > 59 || $longSec1 < 0){
	    push(@invalidParams, "Invalid first longitude seconds specified.");
	}

	if(hasContent($FORM::latDeg2) && ($FORM::latDeg2 > 90 || $FORM::latDeg2 < 0)){
	    push(@invalidParams, "Invalid second latitude degrees specified.");
	}
	if(hasContent($FORM::latMin2) && ($FORM::latMin2 > 59 || $FORM::latMin2 < 0)){
	    push(@invalidParams, "Invalid second latitude minutes specified.");
	}
	if(hasContent($FORM::latSec2) && ($FORM::latSec2 > 59 || $FORM::latSec2 < 0)){
	    push(@invalidParams, "Invalid second latitude seconds specified.");
	}
	if(hasContent($FORM::latDeg2) && ($FORM::longDeg2 > 180 || $FORM::longDeg2 < 0)){
	    push(@invalidParams, "Invalid second longitude degrees specified.");
	}
	if(hasContent($FORM::latMin2) && ($FORM::longMin2 > 59 || $FORM::longMin2 < 0)){
	    push(@invalidParams, "Invalid second longitude minutes specified.");
	}
	if(hasContent($FORM::latSec2) && ($FORM::longSec2 > 59 || $FORM::longSec2 < 0)){
	    push(@invalidParams, "Invalid second longitude seconds specified.");
	}
    }
    
    # Check if latDeg1 and longDeg1 has values if useSiteCoord is used. 
    # This check is required because some of the sites dont have lat 
    # and long mentioned in the config file. 


    if($FORM::hasSpatial eq 'true'){
	if ($FORM::useSiteCoord ) {
	    push(@invalidParams, "The Data Registry doesn't have latitude and longitude information for the site that you chose. Please go back and enter the spatial information.")
		unless(hasContent($latDeg1) && hasContent($longDeg1));
	}else{
	    push(@invalidParams, "Latitude degrees are missing.")
		unless (hasContent($latDeg1) || $FORM::spatialRequired ne 'true');
	    push(@invalidParams, "Longitude degrees are missing.")
		unless (hasContent($longDeg1) || $FORM::spatialRequired ne 'true');
	}
	push(@invalidParams, 
	     "You must provide a geographic description if you provide latitude and longitude information.")
	    if ((hasContent($latDeg1) || (hasContent($longDeg1))) && (!hasContent($FORM::geogdesc)));
    }

    if($FORM::hasMethod eq 'true'){
	push(@invalidParams, 
	     "You must provide a method description if you provide a method title.")
	    if (hasContent($FORM::methodTitle) && ( !(scalar(@FORM::methodPara) > 0) 
						    || (! hasContent($FORM::methodPara[0]))));
	push(@invalidParams, 
	     "You must provide a method description if you provide a study extent description.")
	    if (hasContent($FORM::studyExtentDescription) && (!(scalar(@FORM::methodPara) > 0) 
							      || (! hasContent($FORM::methodPara[0]))));
	push(@invalidParams, 
	     "You must provide both a study extent description and a sampling description, or neither.")
	    if (
                (hasContent($FORM::studyExtentDescription) && !hasContent($FORM::samplingDescription)) ||
                (!hasContent($FORM::studyExtentDescription) && hasContent($FORM::samplingDescription))
		);
    }

    push(@invalidParams, "Contact first name is missing.")
    unless (hasContent($FORM::origNamefirstContact) || 
        $FORM::useOrigAddress);
    push(@invalidParams, "Contact last name is missing.")
    unless (hasContent($FORM::origNamelastContact) || 
        $FORM::useOrigAddress);
    push(@invalidParams, "Data medium is missing.")
    unless (hasContent($FORM::dataMedium) || $FORM::dataMedium =~ /elect/);
    
    return \@invalidParams;
}

################################################################################
# 
# utility function to determine if a paramter is defined and not an empty string
#
################################################################################
sub hasContent {
    my $param = shift;

    my $paramHasContent;
    if (!defined($param) || $param eq '') { 
        $paramHasContent = 0;
    } else {
        $paramHasContent = 1;
    }
    return $paramHasContent;
}

################################################################################
#
# Subroutine for replacing characters not recognizable by XML and otherwise. 
#
################################################################################
sub normalize{
    my $val = shift;

    $val =~ s/&/&amp;/g;

    $val =~ s/</&lt;/g;
    $val =~ s/>/&gt;/g;
    $val =~ s/\"/&quot;/g;
    
    my $returnVal = "";
    
    foreach (split(//,$val)){
	my $var = unpack "C*", $_; 
	
	if($var<128 && $var>31){
	    $returnVal=$returnVal.$_;
	} elsif ($var<32){
	    if($var == 10){
		$returnVal=$returnVal.$_;
	    }
	    if($var == 13){
		$returnVal=$returnVal.$_;
	    }
	    if($var == 9){
		$returnVal=$returnVal.$_;
	    }
	} else { 
	    $returnVal=$returnVal."&#".$var.";";
	}
    }
    
    $returnVal =~ s/&/%26/g;    
    return $returnVal;
}


################################################################################
#
# Subroutine for replacing characters that might create problem in HTML. 
# Specifically written for " being used in any text field. This create a 
# problem in confirmData template, when you specify input name value pair 
# with value having a " in it.  
#
################################################################################
sub normalizeCD{
    my $val = shift;

    $val =~ s/\"/&quot;/g;
    
    return $val;
}


################################################################################
# 
# Create the XML document from the HTML form input
# returns the XML document as a string
#
################################################################################
sub createXMLDocument {

    my $orig  = "";
    my $role  = "associatedParty";
    my $creat = "";
    my $metaP = "";
    my $apart = "";
    my $cont  = "";
    my $publ  = "";
    my $dso   = "";
    my $gmt = gmtime($now);


    my $doc =  "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n";

    $doc .= "<eml:eml\n 
                     \t packageId=\"docid\" system=\"knb\"\n 
                     \t xmlns:eml=\"eml://ecoinformatics.org/eml-2.0.1\"\n
                     \t xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n 
                     \t xmlns:ds=\"eml://ecoinformatics.org/dataset-2.0.1\"\n 
                     \t xmlns:stmml=\"http://www.xml-cml.org/schema/stmml\"\n 
                     \t xsi:schemaLocation=\"eml://ecoinformatics.org/eml-2.0.1 eml.xsd\">\n";

    $doc .= "<!-- Person who filled in the catalog entry form: ";
    $doc .= normalize($FORM::providerGivenName)." ".normalize($FORM::providerSurName)." -->\n";
    $doc .= "<!-- Form filled out at $gmt GMT -->\n";
    $doc .= "<dataset>\n";
    
    if (hasContent($FORM::identifier)) {
        $doc .= "<alternateIdentifier system=\"$FORM::site\">";
        $doc .= normalize($FORM::identifier) . "</alternateIdentifier>\n";
    }
    
    if (hasContent($FORM::title)) {
        $doc .= "<title>".normalize($FORM::title)."</title>\n";
    }

    if (hasContent($FORM::origNamelast0)) {
    $role = "creator";
        $orig .= "<individualName>\n";
        $orig .= "<givenName>".normalize($FORM::origNamefirst0)."</givenName>\n";
        $orig .= "<surName>".normalize($FORM::origNamelast0)."</surName>\n";
        $orig .= "</individualName>\n";
    }

    if (hasContent($FORM::origNameOrg)) {
        $orig .= "<organizationName>".normalize($FORM::origNameOrg)."</organizationName>\n";
    }

    if (hasContent($FORM::origDelivery) || hasContent($FORM::origCity) ||
        (hasContent($FORM::origState   ) &&
        ($FORM::origState !~ "Select state here.")) ||
        hasContent($FORM::origStateOther) ||
        hasContent($FORM::origZIP ) || hasContent($FORM::origCountry)) {
        $orig .= "<address>\n";

        if (hasContent($FORM::origDelivery)) {
            $orig .= "<deliveryPoint>".normalize($FORM::origDelivery)."</deliveryPoint>\n";
        }
        if (hasContent($FORM::origCity)) {
            $orig .= "<city>".normalize($FORM::origCity)."</city>\n";
        }

    if (hasContent($FORM::origState) && 
            ($FORM::origState !~ "Select state here.")) {
            $orig .= "<administrativeArea>".normalize($FORM::origState);
            $orig .= "</administrativeArea>\n";
        } elsif (hasContent($FORM::origStateOther)) {
            $orig .= "<administrativeArea>".normalize($FORM::origStateOther);
            $orig .= "</administrativeArea>\n";
        }
        if (hasContent($FORM::origZIP)) {
            $orig .= "<postalCode>".normalize($FORM::origZIP)."</postalCode>\n";
        }
        if (hasContent($FORM::origCountry)) {
            $orig .= "<country>".normalize($FORM::origCountry)."</country>\n";
        }
        $orig .= "</address>\n";
    }

    if (hasContent($FORM::origPhone)) {
        $orig .= "<phone>".normalize($FORM::origPhone)."</phone>\n";
    }
    if (hasContent($FORM::origFAX)) {
        $orig .= "<phone phonetype=\"Fax\">".normalize($FORM::origFAX)."</phone>\n";
    }
    if (hasContent($FORM::origEmail)) {
        $orig .= "<electronicMailAddress>".normalize($FORM::origEmail);
        $orig .= "</electronicMailAddress>\n";
    }
    $dso = "<$role>\n$orig</$role>\n";
    
    if ($FORM::cfg eq 'nceas') {
        for (my $i = 0; $i < scalar(@FORM::wg); $i++) {
            $creat .= "<creator>\n";
            $creat .= "<organizationName>".normalize($FORM::wg[$i])."</organizationName>\n";
            $creat .= "</creator>\n";
        }
    } else {
	    $creat .= "<creator>\n";
	    $creat .= "<organizationName>".normalize($FORM::site)."</organizationName>\n";
	    $creat .= "</creator>\n";
    }

    if ($FORM::cfg ne 'knb') {
        $creat .= "<creator>\n";
        $creat .= "<organizationName>".normalize($organization)."</organizationName>\n";
        $creat .= "</creator>\n";
    }

    $creat .= $dso;

    if ($FORM::useOrigAddress) {
        # Add a contact originator like the original with a different role
            $cont .= "<contact>\n";
        $cont .= $orig;
        $cont .= "</contact>\n";
    } else {
        $cont .= "<contact>\n";

        $cont .= "<individualName>\n";
        $cont .= "<givenName>".normalize($FORM::origNamefirstContact)."</givenName>\n";
        $cont .= "<surName>".normalize($FORM::origNamelastContact)."</surName>\n";
        $cont .= "</individualName>\n";
 
    if (hasContent($FORM::origNameOrgContact)) {
        $cont .= "<organizationName>".normalize($FORM::origNameOrgContact)."</organizationName>\n";
    }

        if (hasContent($FORM::origDeliveryContact) || 
            hasContent($FORM::origCityContact) ||
            (hasContent($FORM::origStateContact) &&
            ($FORM::origStateContact !~ "Select state here.")) ||
            hasContent($FORM::origStateOtherContact) ||
            hasContent($FORM::origZIPContact) || 
            hasContent($FORM::origCountryContact)) {
            $cont .= "<address>\n";
            if (hasContent($FORM::origDeliveryContact)) {
                $cont .= "<deliveryPoint>".normalize($FORM::origDeliveryContact);
                $cont .= "</deliveryPoint>\n";
            }
            if (hasContent($FORM::origCityContact)) {
                $cont .= "<city>".normalize($FORM::origCityContact)."</city>\n";
            }
            if (hasContent($FORM::origStateContact) && 
                ($FORM::origStateContact !~ "Select state here.")) {
                $cont .= "<administrativeArea>".normalize($FORM::origStateContact);
                $cont .= "</administrativeArea>\n";
            } elsif (hasContent($FORM::origStateOtherContact)) {
                $cont .= "<administrativeArea>".normalize($FORM::origStateOtherContact);
                $cont .= "</administrativeArea>\n";
            }
            if (hasContent($FORM::origZIPContact)) {
                $cont .= "<postalCode>".normalize($FORM::origZIPContact)."</postalCode>\n";
            }
            if (hasContent($FORM::origCountryContact)) {
                $cont .= "<country>".normalize($FORM::origCountryContact)."</country>\n";
            }
            $cont .= "</address>\n";
        }
        if (hasContent($FORM::origPhoneContact)) {
            $cont .= "<phone>".normalize($FORM::origPhoneContact)."</phone>\n";
        }
    if (hasContent($FORM::origFAXContact)) {
        $cont .= "<phone phonetype=\"Fax\">".normalize($FORM::origFAXContact)."</phone>\n";
    }
        if (hasContent($FORM::origEmailContact)) {
            $cont .= "<electronicMailAddress>".normalize($FORM::origEmailContact);
            $cont .= "</electronicMailAddress>\n";
        }
    $cont .= "</contact>\n";
    }

    $metaP .= "<metadataProvider>\n";
    $metaP .= "<individualName>\n";
    $metaP .= "<givenName>".normalize($FORM::providerGivenName)."</givenName>\n";
    $metaP .= "<surName>".normalize($FORM::providerSurName)."</surName>\n";
    $metaP .= "</individualName>\n";
    $metaP .= "</metadataProvider>\n";

    # Additional originators
    foreach my $tmp (param()) {
        if ($tmp =~ /origNamelast/){
            my $tmp1 = $tmp;
            $tmp1 =~ s/origNamelast//; # get the index of the parameter 0 to 10
            if ( $tmp1 eq '1' 
                 || $tmp1 eq '2'
                 || $tmp1 eq '3'
                 || $tmp1 eq '4'
                 || $tmp1 eq '5'
                 || $tmp1 eq '6'
                 || $tmp1 eq '7'
                 || $tmp1 eq '8'
                 || $tmp1 eq '9'
                 || $tmp1 eq '10'
                 ) {
     
                # do not generate XML for empty originator fields 
                if (hasContent(param("origNamefirst" . $tmp1))) {    

            my $add = "";
            $add .= "<individualName>\n";
            $add .= "<givenName>";
            $add .= normalize(param("origNamefirst" . $tmp1));
            $add .= "</givenName>\n";
            $add .= "<surName>";
            $add .= normalize(param("origNamelast" . $tmp1));
            $add .= "</surName>\n";
            $add .= "</individualName>\n";
            
            if(param("origRole" . $tmp1) eq "Originator"){
            $creat .= "<creator>\n";
            $creat .= $add;
            $creat .= "</creator>\n";
            }
            elsif(param("origRole" . $tmp1) eq "Metadata Provider"){
            $metaP .= "<metadataProvider>\n";
            $metaP .= $add;
            $metaP .= "</metadataProvider>\n";
            }
            elsif((param("origRole" . $tmp1) eq "Publisher")  && ($publ eq "")){
            $publ .= "<publisher>\n";
            $publ .= $add;
            $publ .= "</publisher>\n";
            }
            else{
            $apart .= "<associatedParty>\n";
            $apart .= $add;
            $apart .= "<role>";
            $apart .= param("origRole" . $tmp1);
            $apart .= "</role>\n";
            $apart .= "</associatedParty>\n";
            }
        }
            }
        }
    }

    $doc .= $creat;
    $doc .= $metaP;
    $doc .= $apart;

    $doc .= "<abstract>\n";
    $doc .= "<para>".normalize($FORM::abstract)."</para>\n";
    $doc .= "</abstract>\n";

    # Keyword information
    foreach my $tmp (param()) {
        if ($tmp =~ /keyword/) {
            my $tmp1 = $tmp;
            $tmp1 =~ s/keyword//; # get the index of the parameter 0, ..., 10
            if ( $tmp1 =~ /[0-9]/ ){
                # don't generate xml for empty keyword fields
                # don't generate taxonomic keyword fields, those go in taxonomic coverage
                if (hasContent(param($tmp))) {
                    $doc .= "<keywordSet>\n";
                    $doc .= "<keyword ";
                    if (hasContent(param("kwType" . $tmp1)) &&
                       (param("kwType" . $tmp1) !~ "none") ) {
                         $doc .= "keywordType=\"";
                         $doc .= param("kwType" . $tmp1);
                         $doc .= "\"";
                    }
                    $doc .= ">";
                    $doc .= normalize(param("keyword" . $tmp1));
                    $doc .= "</keyword>\n";
                    $doc .= "<keywordThesaurus>";
                    $doc .= normalize(param("kwTh" . $tmp1));
                    $doc .= "</keywordThesaurus>\n";
                    $doc .= "</keywordSet>\n";
                }
            }
        }
    }

    if (hasContent($FORM::addComments)) {
        $doc .= "<additionalInfo>\n";
        $doc .= "<para>".normalize($FORM::addComments)."</para>\n";
        $doc .= "</additionalInfo>\n";
    }

    if (hasContent($FORM::useConstraints) || 
        hasContent($FORM::useConstraintsOther)) {
        $doc .= "<intellectualRights>\n";
        if (hasContent($FORM::useConstraints)) {
            $doc .= "<para>".normalize($FORM::useConstraints)."</para>\n";
        }
        if (hasContent($FORM::useConstraintsOther)) {
            $doc .= "<para>".normalize($FORM::useConstraintsOther)."</para>\n";
        }
        $doc .= "</intellectualRights>\n";
    }

    
    if (hasContent($FORM::url)) {
    $doc .= "<distribution>\n";
        $doc .= "<online>\n";
    $doc .= "<url>".normalize($FORM::url)."</url>\n";
    $doc .= "</online>\n";
    $doc .= "</distribution>\n";
    }
    
    $doc .= "<distribution>\n";
    $doc .= "<offline>\n";
    $doc .= "<mediumName>" . normalize($FORM::dataMedium)." ".normalize($FORM::dataMediumOther);
    $doc .= "</mediumName>\n";
    $doc .= "</offline>\n";
    $doc .= "</distribution>\n";
            
    my $cov = "";

    if (hasContent($FORM::endingYear)) {
	$cov .= "<temporalCoverage>\n";
	$cov .= "<rangeOfDates>\n";
	if (hasContent($FORM::beginningMonth)) {
	    my $month = ("JAN","FEB","MAR","APR","MAY","JUN",
			 "JUL","AUG","SEP","OCT","NOV","DEC")
		[$FORM::beginningMonth - 1];
	    $cov .= "<beginDate>\n";
	    $cov .= "<calendarDate>";
	    $cov .= normalize($FORM::beginningYear)."-".normalize($FORM::beginningMonth)."-".normalize($FORM::beginningDay);
	    $cov .= "</calendarDate>\n";
	    $cov .= "</beginDate>\n";
	} else {
	    $cov .= "<beginDate>\n";
	    $cov .= "<calendarDate>";
	    $cov .= normalize($FORM::beginningYear);
	    $cov .= "</calendarDate>\n";
	    $cov .= "</beginDate>\n";
	}
	
	if (hasContent($FORM::endingMonth)) {
	    my $month = ("JAN","FEB","MAR","APR","MAY","JUN",
			 "JUL","AUG","SEP","OCT","NOV","DEC")
		[$FORM::endingMonth - 1];
	    $cov .= "<endDate>\n";
	    $cov .= "<calendarDate>";
	    $cov .= normalize($FORM::endingYear)."-".normalize($FORM::endingMonth)."-".normalize($FORM::endingDay);
	    $cov .= "</calendarDate>\n";
	    $cov .= "</endDate>\n";
	} else {
	    $cov .= "<endDate>\n";
	    $cov .= "<calendarDate>";
	    $cov .= normalize($FORM::endingYear);
	    $cov .= "</calendarDate>\n";
	    $cov .= "</endDate>\n";
	}
	$cov .= "</rangeOfDates>\n";
	$cov .= "</temporalCoverage>\n";
    } else {
	if(hasContent($FORM::beginningYear)) {
	    $cov .= "<temporalCoverage>\n";
	    $cov .= "<singleDateTime>\n";
	    if (hasContent($FORM::beginningMonth)) {
		my $month = ("JAN","FEB","MAR","APR","MAY","JUN",
			     "JUL","AUG","SEP","OCT","NOV","DEC")
		    [$FORM::beginningMonth - 1];
		$cov .= "<calendarDate>";
		$cov .= normalize($FORM::beginningYear)."-".normalize($FORM::beginningMonth)."-".normalize($FORM::beginningDay);
		$cov .= "</calendarDate>\n";
	    } else {
		$cov .= "<calendarDate>";
		$cov .= normalize($FORM::beginningYear);
		$cov .= "</calendarDate>\n";
	    }
	    $cov .= "</singleDateTime>\n";
	    $cov .= "</temporalCoverage>\n";
	}
    }
    
    if(hasContent($FORM::geogdesc) || ($FORM::latDeg1 != 0 && $FORM::longDeg1 != 0)) {
	$cov .= "<geographicCoverage>\n";

	if(hasContent($FORM::geogdesc)) {
	    $cov .= "<geographicDescription>".normalize($FORM::geogdesc)."</geographicDescription>\n";
	}
	
	if($latDeg1 != 0 && $longDeg1 != 0) {
	    $cov .= "<boundingCoordinates>\n";
	    # if the second latitude is missing, then set the second lat/long pair 
	    # equal to the first this makes a point appear like a rectangle 
	    if ($FORM::useSiteCoord || ($FORM::latDeg2 == 0 && $FORM::latMin2 == 0 && $FORM::latSec2 == 0)) {
		
		$latDeg2 = $latDeg1;
		$latMin2 = $latMin1;
		$latSec2 = $latSec1;
		$hemisphLat2 = $hemisphLat1;
		$longDeg2 = $longDeg1;
		$longMin2 = $longMin1;
		$longSec2 = $longSec1;
		$hemisphLong2 = $hemisphLong1;
	    }
	    else
	    {
		$latDeg2 = $FORM::latDeg2;
		$latMin2 = $FORM::latMin2;
		$latSec2 = $FORM::latSec2;
		$hemisphLat2 = $FORM::hemisphLat2;
		$longDeg2 = $FORM::longDeg2;
		$longMin2 = $FORM::longMin2;
		$longSec2 = $FORM::longSec2;
		$hemisphLong2 = $FORM::hemisphLong2;
	    } 
	
	
	    my $hemisph;
	    $hemisph = ($hemisphLong1 eq "W") ? -1 : 1;
	    $cov .= "<westBoundingCoordinate>";
	    my $var = $hemisph * ($longDeg1 + (60*$longMin1+$longSec1)/3600);
	    $cov .= sprintf("%.4f\n", $var);
	    $cov .= "</westBoundingCoordinate>\n";
	    
	    $hemisph = ($hemisphLong2 eq "W") ? -1 : 1;
	    $cov .= "<eastBoundingCoordinate>";
	    $var = $hemisph * ($longDeg2 + (60*$longMin2+$longSec2)/3600);
	    $cov .= sprintf("%.4f\n", $var);
	    $cov .= "</eastBoundingCoordinate>\n";
	    
	    $hemisph = ($hemisphLat1 eq "S") ? -1 : 1;
	    $cov .= "<northBoundingCoordinate>";
	    $var = $hemisph * ($latDeg1 + (60*$latMin1+$latSec1)/3600);
	    $cov .= sprintf("%.4f\n", $var);	   
	    $cov .= "</northBoundingCoordinate>\n";
	    
	    $hemisph = ($hemisphLat2 eq "S") ? -1 : 1;
	    $cov .= "<southBoundingCoordinate>";
	    $var = $hemisph * ($latDeg2 + (60*$latMin2+$latSec2)/3600);
	    $cov .= sprintf("%.4f\n", $var);
	    $cov .= "</southBoundingCoordinate>\n";
	    
	    $cov .= "</boundingCoordinates>\n";
	}
	$cov .= "</geographicCoverage>\n";
    }

    # Write out the taxonomic coverage fields
    my $foundFirstTaxon = 0;
    foreach my $trn (param()) {
        if ($trn =~ /taxonRankName/) {
            my $taxIndex = $trn;
            $taxIndex =~ s/taxonRankName//; # get the index of the parameter 0, ..., 10
            my $trv = "taxonRankValue".$taxIndex;
            if ( $taxIndex =~ /[0-9]/ ){
                if (hasContent(param($trn)) && hasContent(param($trv))) {
                    if (! $foundFirstTaxon) {
                        $cov .= "<taxonomicCoverage>\n";
                        $foundFirstTaxon = 1;
                        if (hasContent($FORM::taxaAuth)) {
                            $cov .= "<generalTaxonomicCoverage>".normalize($FORM::taxaAuth)."</generalTaxonomicCoverage>\n";
                        }
                    }
                    $cov .= "<taxonomicClassification>\n";
                    $cov .= "  <taxonRankName>".normalize(param($trn))."</taxonRankName>\n";
                    $cov .= "  <taxonRankValue>".normalize(param($trv))."</taxonRankValue>\n";
                    $cov .= "</taxonomicClassification>\n";
                }
            }
        }
    }
    if ($foundFirstTaxon) {
        $cov .= "</taxonomicCoverage>\n";
    }

    if($cov ne "" ){
	$doc .= "<coverage>".$cov."</coverage>";
    }
    $doc .= $cont;
    $doc .= $publ;
    
    if ((hasContent($FORM::methodTitle)) || scalar(@FORM::methodsPara) > 0 || ($FORM::methodPara[0] ne "")) {
        my $methods = "<methods><methodStep><description><section>\n";
        if (hasContent($FORM::methodTitle)) {
            $methods .= "<title>".normalize($FORM::methodTitle)."</title>\n";
        }
        for (my $i = 0; $i < scalar(@FORM::methodPara); $i++) {
            $methods .= "<para>".normalize($FORM::methodPara[$i])."</para>\n";
        }
        $methods .= "</section></description></methodStep>\n";
        if (hasContent($FORM::studyExtentDescription)) {
            $methods .= "<sampling><studyExtent><description>\n";
            $methods .= "<para>".normalize($FORM::studyExtentDescription)."</para>\n";
            $methods .= "</description></studyExtent>\n";
            $methods .= "<samplingDescription>\n";
            $methods .= "<para>".normalize($FORM::samplingDescription)."</para>\n";
            $methods .= "</samplingDescription>\n";
            $methods .= "</sampling>\n";
        }
        $methods .= "</methods>\n";
        $doc .= $methods;
    }

    $doc .= "<access authSystem=\"knb\" order=\"denyFirst\">\n";
    $doc .= "<allow>\n";
    $doc .= "<principal>$username</principal>\n";
    $doc .= "<permission>all</permission>\n";
    $doc .= "</allow>\n";
    $doc .= "<allow>\n";
    $doc .= "<principal>uid=$FORM::username,o=$FORM::organization,dc=ecoinformatics,dc=org</principal>\n";
    $doc .= "<permission>all</permission>\n";
    $doc .= "</allow>\n";
    $doc .= "<allow>\n";
    $doc .= "<principal>public</principal>\n";
    $doc .= "<permission>read</permission>\n";
    $doc .= "</allow>\n";
    $doc .= "</access>\n";
    
    $doc .= "</dataset>\n</eml:eml>\n";

    return $doc;
}


################################################################################
# 
# send an email message notifying the moderator of a new submission 
#
################################################################################
sub sendNotification {
    my $identifier = shift;
    my $mailhost = shift;
    my $sender = shift;
    my $recipient = shift;

    my $smtp = Net::SMTP->new($mailhost);
    $smtp->mail($sender);
    $smtp->to($recipient);

    my $message = <<"    ENDOFMESSAGE";
    To: $recipient
    From: $sender
    Subject: New data submission
    
    Data was submitted to the data registry.  
    The identifying information for the new data set is:

    Identifier: $identifier
    Title: $FORM::title
    Submitter: $FORM::providerGivenName $FORM::providerSurName

    Please review the submmission and grant public read access if appropriate.
    Thanks
    
    ENDOFMESSAGE
    $message =~ s/^[ \t\r\f]+//gm;

    $smtp->data($message);
    $smtp->quit;
}


################################################################################
# 
# read the eml document and send back a form with values filled in. 
#
################################################################################
sub modifyData {
    
    # create metacat instance
    my $metacat;
    my $docid = $FORM::docid;
    my $httpMessage;
    my $doc;
    my $xmldoc;
    my $findType;
    my $parser = XML::LibXML->new();
    my @fileArray;
    my $pushDoc;
    my $alreadyInArray;
    my $node;
    my $response; 
    my $element;
    my $tempfile;

    $metacat = Metacat->new();
    if ($metacat) {
        $metacat->set_options( metacatUrl => $metacatUrl );
    } else {
        #die "failed during metacat creation\n";
        push(@errorMessages, "Failed during metacat creation.");
    }
    
    $httpMessage = $metacat->read($docid);
    $doc = $httpMessage->content();
    $xmldoc = $parser->parse_string($doc);

    #$tempfile = $xslConvDir.$docid;
    #push (@fileArray, $tempfile);

    if ($xmldoc eq "") {
        $error ="Error in parsing the eml document";
        push(@errorMessages, $error);
    } else {
        $findType = $xmldoc->findnodes('//dataset/identifier');
        if ($findType->size() > 0) {
            # This is a eml beta6 document
            # Read the documents mentioned in triples also
        
            $findType = $xmldoc->findnodes('//dataset/triple');
            if ($findType->size() > 0) {
                foreach $node ($findType->get_nodelist) {
                    $pushDoc = findValue($node, 'subject');
            
                    # If the file is already in @fileArray then do not add it 
                    $alreadyInArray = 0;
                    foreach $element (@fileArray) {
                        $tempfile = $tmpdir."/".$pushDoc;
                        if ($element eq $pushDoc) {
                            $alreadyInArray = 1;
                        }
                    }
            
                    if (!$alreadyInArray) {
                        $tempfile = $tmpdir."/".$pushDoc;
                        $response = "";
                        $response = $metacat->read($pushDoc);    
                        if (! $response) {
                            # could not read
                            #push(@errorMessages, $response);
                            push(@errorMessages, $metacat->getMessage());
                            push(@errorMessages, "Failed during reading.\n");
                        } else {
                            my $xdoc = $response->content();
                            #$tempfile = $xslConvDir.$pushDoc;
                            open (TFILE,">$tempfile") || 
                                die ("Cant open xml file... $tempfile\n");
                            print TFILE $xdoc;
                            close(TFILE);
                            push (@fileArray, $tempfile);
                        }
                    }
                }
            }

            # Read the main document. 

            $tempfile = $tmpdir."/".$docid; #= $xslConvDir.$docid;
            open (TFILE,">$tempfile") || die ("Cant open xml file...\n");
            print TFILE $doc;
            close(TFILE);
        
            # Transforming beta6 to eml 2
            my $xslt;
            my $triplesheet;
            my $results;
            my $stylesheet;
            my $resultsheet;
        
            $xslt = XML::LibXSLT->new();
            #$tempfile = $xslConvDir."triple_info.xsl";
            $tempfile = $tmpdir."/"."triple_info.xsl";
    
            $triplesheet = $xslt->parse_stylesheet_file($tempfile);

            #$results = $triplesheet->transform($xmldoc, packageDir => "\'$tmpdir/\'", packageName => "\'$docid\'");
            $results = $triplesheet->transform($xmldoc, packageDir => "\'$tmpdir/\'", packageName => "\'$docid\'");

            #$tempfile = $xslConvDir."emlb6toeml2.xsl";
            $tempfile = $tmpdir."/"."emlb6toeml2.xsl";
            $stylesheet = $xslt->parse_stylesheet_file($tempfile);
            $resultsheet = $stylesheet->transform($results);
        
            #$tempfile = "/usr/local/apache2/htdocs/xml/test.xml";;
            #open (TFILE,">$tempfile") || die ("Cant open xml file...\n");
            #print TFILE $stylesheet->output_string($resultsheet);
            #close(TFILE);

            getFormValuesFromEml2($resultsheet);
            
            # Delete the files written earlier. 
            unlink @fileArray;

        } else {
            getFormValuesFromEml2($xmldoc);
        }
    }   
    
    if (scalar(@errorMessages)) {
        # if any errors, print them in the response template 
        $$templateVars{'status'} = 'failure_no_resubmit';
        $$templateVars{'errorMessages'} = \@errorMessages;
        $error = 1;
        $$templateVars{'function'} = "modification";
        $$templateVars{'section'} = "Modification Status";
        $template->process( $responseTemplate, $templateVars); 
    } else {
        $$templateVars{'form'} = 're_entry';
        $template->process( $entryFormTemplate, $templateVars);
    }
}

################################################################################
# 
# Parse an EML 2.0.0 file and extract the metadata into perl variables for 
# processing and returning to the template processor
#
################################################################################
sub getFormValuesFromEml2 {
    
    my $doc = shift;
    my $results;
    my $error;
    my $node;
    my $tempResult;
    my $tempNode;
    my $aoCount = 1;
    my $foundDSO;

    # set variable values
    $$templateVars{'showSiteList'} = $showSiteList;
    $$templateVars{'lsite'} = $lsite;
    $$templateVars{'usite'} = $usite;
    $$templateVars{'showWgList'} = $showWgList;
    $$templateVars{'showOrganization'} = $showOrganization;
    $$templateVars{'hasKeyword'} = $hasKeyword;
    $$templateVars{'hasTemporal'} = $hasTemporal;
    $$templateVars{'hasSpatial'} = $hasSpatial;
    $$templateVars{'hasTaxonomic'} = $hasTaxonomic;
    $$templateVars{'hasMethod'} = $hasMethod;
    $$templateVars{'spatialRequired'} = $spatialRequired;
    $$templateVars{'temporalRequired'} = $temporalRequired;

    # find out the tag <alternateIdentifier>. 
    $results = $doc->findnodes('//dataset/alternateIdentifier');
    if ($results->size() > 1) {
        errMoreThanOne("alternateIdentifier");
    } else {
        foreach $node ($results->get_nodelist) {
            $$templateVars{'identifier'} = findValue($node, '../alternateIdentifier');
        }
    }

    # find out the tag <title>. 
    $results = $doc->findnodes('//dataset/title');
    if ($results->size() > 1) {
        errMoreThanOne("title");
    } elsif ($results->size() < 1) {
        $error ="Following tag not found: title. Please use Morpho to edit this document";
        push(@errorMessages, $error."\n");
        #if ($DEBUG == 1){ print $error;}
    } else {
        foreach $node ($results->get_nodelist) {
            $$templateVars{'title'} = findValue($node, '../title');
        }
    }

    # find out the tag <creator>. 
    $results = $doc->findnodes('//dataset/creator/individualName');
    debug("Registry: Creators: ".$results->size());
     foreach $node ($results->get_nodelist) {
            dontOccur($node, "../positionName|../onlineURL|../userId", 
              "positionName, onlineURL, userId");
        
            dontOccur($node, "./saluation", "saluation");                
        
            debug("Registry: Checking a creator in loop 1...");
            $tempResult = $node->findnodes('../address|../phone|../electronicmailAddress|../organizationName');
            if($tempResult->size > 0) {
                if($foundDSO == 0) {
                    $foundDSO = 1;
     
                    debug("Registry: Recording a creator in loop 1...");
                    $$templateVars{'origNamefirst0'} = findValue($node, 'givenName');
                    $$templateVars{'origNamelast0'} = findValue($node, 'surName');
            
                    my $tempResult2 = $node->findnodes('../address');
                    if ($tempResult2->size > 1) {
                        errMoreThanOne("address");
                    } else {
                        foreach my $tempNode2 ($tempResult2->get_nodelist) {
                            $$templateVars{'origDelivery'} = findValue($tempNode2, 'deliveryPoint');
                            $$templateVars{'origCity'} = findValue($tempNode2, 'city');
                            $$templateVars{'origState'} = findValue($tempNode2, 'administrativeArea');
                            $$templateVars{'origZIP'} = findValue($tempNode2, 'postalCode');
                            $$templateVars{'origCountry'} = findValue($tempNode2, 'country');
                        }
                    }
            
                    my $tempResult3 = $node->findnodes('../phone');
                    if ($tempResult3->size > 2) {
                        errMoreThanN("phone");
                    } else {
                        foreach my $tempNode2 ($tempResult3->get_nodelist) {
                            if ($tempNode2->hasAttributes()) {
                                my @attlist = $tempNode2->attributes();
                                if ($attlist[0]->value eq "Fax") {
                                    $$templateVars{'origFAX'} = $tempNode2->textContent();
                                } else {
                                    $$templateVars{'origPhone'} = $tempNode2->textContent();
                                }
                            } else {
                                $$templateVars{'origPhone'} = $tempNode2->textContent();
                            }
                        }
                    }
                    $$templateVars{'origEmail'} = findValue($node, '../electronicMailAddress');
                    $$templateVars{'origNameOrg'} = findValue($node, '../organizationName');
                } else {
                    errMoreThanN("address, phone and electronicMailAddress");
                }
            }
        }
        foreach $node ($results->get_nodelist) {
            debug("Registry: Checking a creator in loop 2...");
            $tempResult = $node->findnodes('../address|../phone|../electronicmailAddress|../organizationName');
            if ($tempResult->size == 0) {
                if ($foundDSO == 0) {
                    debug("Registry: Recording a creator in loop 2 block A...");
                    $foundDSO = 1;
                    $$templateVars{'origNamefirst0'} = findValue($node, 'givenName');
                    $$templateVars{'origNamelast0'} = findValue($node, 'surName');
                    $$templateVars{'origNameOrg'} = findValue($node, '../organizationName');
                } else {
                    debug("Registry: Recording a creator in loop 2 block B...");
                    $$templateVars{"origNamefirst$aoCount"} =  findValue($node, './givenName');
                    $$templateVars{"origNamelast$aoCount"} =  findValue($node, './surName');
                    $$templateVars{"origRole$aoCount"} = "Originator";
                    $aoCount++;
                }
            }
        }

    $results = $doc->findnodes('//dataset/creator/organizationName');
    my $wgroups = $doc->findnodes("//dataset/creator/organizationName[contains(text(),'(NCEAS ')]");
    debug("Registry: Number Org: ".$results->size());
    debug("Registry:  Number WG: ".$wgroups->size());
    if ($results->size() - $wgroups->size() > 3) {
        errMoreThanN("creator/organizationName");    
    } else {
        foreach $node ($results->get_nodelist) {
            my $tempValue = findValue($node,'../organizationName');
            $tempResult = $node->findnodes('../individualName');
            if ($tempResult->size == 0 && $tempValue ne $organization) {
                $$templateVars{'site'} = $tempValue;
            }
        }
        if ($FORM::cfg eq 'nceas') {
            my @wg;
            foreach $node ($results->get_nodelist) {
                my $tempValue = findValue($node,'../organizationName');
                $wg[scalar(@wg)] = $tempValue;
            }
            my $projects = getProjectList();
            $$templateVars{'projects'} = $projects;
            $$templateVars{'wg'} = \@wg;
        }
    }

    $results = $doc->findnodes('//dataset/metadataProvider');
    foreach $node ($results->get_nodelist) {
            dontOccur($node, "./organizationName|./positionName|./onlineURL|./userId|./electronicMailAddress|./phone|./address", 
                "organizationName, positionName, onlineURL, userId, electronicMailAddress, phone, address in metadataProvider");
        
	    $tempResult = $node->findnodes('./individualName');
            if ($tempResult->size > 1) {
                errMoreThanOne("metadataProvider/indvidualName");
            } else {
                foreach $tempNode ($tempResult->get_nodelist) {
                    if ($$templateVars{'providerGivenName'} ne "") {
                        $$templateVars{"origNamefirst$aoCount"} =  findValue($tempNode, './givenName');
                        $$templateVars{"origNamelast$aoCount"} =  findValue($tempNode, './surName');
                        $$templateVars{"origRole$aoCount"} = "Metadata Provider";
                        $aoCount++;
                    } else {
                        $$templateVars{'providerGivenName'} =  findValue($tempNode, './givenName');
                        $$templateVars{'providerSurName'} =  findValue($tempNode, './surName');
                    }
                }
            }
        }

    $results = $doc->findnodes('//dataset/associatedParty');
    foreach $node ($results->get_nodelist) {
            dontOccur($node, "./organizationName|./positionName|./onlineURL|./userId|./electronicMailAddress|./phone|./address", 
                "organizationName, positionName, onlineURL, userId, electronicMailAddress, phone, address in associatedParty");
       
            $tempResult = $node->findnodes('./individualName');
            if ($tempResult->size > 1) {
                errMoreThanOne("associatedParty/indvidualName");
            } else {
                foreach $tempNode ($tempResult->get_nodelist) {
                    $$templateVars{"origNamefirst$aoCount"} =  findValue($tempNode, './givenName');
                    $$templateVars{"origNamelast$aoCount"} =  findValue($tempNode, './surName');
                    $$templateVars{"origRole$aoCount"} = findValue($tempNode, '../role');
                    $aoCount++;           
                }
            }
     }

    $results = $doc->findnodes('//dataset/publisher');
#    if ($results->size() > 10) {
 #       errMoreThanN("publisher");
 #   } else {
        foreach $node ($results->get_nodelist) {
            dontOccur($node, "./organizationName|./positionName|./onlineURL|./userId|./electronicMailAddress|./phone|./address", 
                "organizationName, positionName, onlineURL, userId, electronicMailAddress, phone, address in associatedParty");
       
            $tempResult = $node->findnodes('./individualName');
            if ($tempResult->size > 1) {
                errMoreThanOne("publisher/indvidualName");
            } else {
                foreach $tempNode ($tempResult->get_nodelist) {
                    $$templateVars{"origNamefirst$aoCount"} =  findValue($tempNode, './givenName');
                    $$templateVars{"origNamelast$aoCount"} =  findValue($tempNode, './surName');
                    $$templateVars{"origRole$aoCount"} = "Publisher";
                    $aoCount++;           
                }
            }
        }
  #  }

  #  if ($aoCount > 11) {
  #      errMoreThanN("Additional Originators");
 #   } 

    $$templateVars{'aoCount'} = $aoCount-1;
    
    dontOccur($doc, "./pubDate", "pubDate");
    dontOccur($doc, "./language", "language");
    dontOccur($doc, "./series", "series");

    $results = $doc->findnodes('//dataset/abstract');
    if ($results->size() > 1) {
        errMoreThanOne("abstract");
    } else {
        foreach my $node ($results->get_nodelist) {
            dontOccur($node, "./section", "section");
            $$templateVars{'abstract'} = findValueNoChild($node, "para");
        }
    }

    $results = $doc->findnodes('//dataset/keywordSet');

    my $count = 0;
    foreach $node ($results->get_nodelist) {
	$tempResult = $node->findnodes('./keyword');
	if ($tempResult->size() > 1) {
	    errMoreThanOne("keyword");
	} else {
	    $count++;
	    foreach $tempNode ($tempResult->get_nodelist) {
		$$templateVars{"keyword$count"} = $tempNode->textContent();
		if ($tempNode->hasAttributes()) {
		    my @attlist = $tempNode->attributes();
		    $$templateVars{"kwType$count"} = $attlist[0]->value;
		}  
	    }
	}
	$$templateVars{"kwTh$count"} = findValue($node, "keywordThesaurus");
    }
    $$templateVars{'keyCount'} = $count;
    if($count > 0 ){
       $$templateVars{'hasKeyword'} = "true";
    }

    $results = $doc->findnodes('//dataset/additionalInfo');
    if ($results->size() > 1) {
        errMoreThanOne("additionalInfo");
    } else {
        foreach $node ($results->get_nodelist) {
            dontOccur($node, "./section", "section");
            $$templateVars{'addComments'} = findValueNoChild($node, "para");
        }
    }

    $$templateVars{'useConstraints'} = "";
    $results = $doc->findnodes('//dataset/intellectualRights');
    if ($results->size() > 1) {
        errMoreThanOne("intellectualRights");
    } else {
        foreach $node ($results->get_nodelist) {
            dontOccur($node, "./section", "section in intellectualRights");

            $tempResult = $node->findnodes("para");
            if ($tempResult->size > 2) {
                   errMoreThanN("para");
            } else {
                foreach $tempNode ($tempResult->get_nodelist) {
                    my $childNodes = $tempNode->childNodes;
                    if ($childNodes->size() > 1) {
                        $error ="The tag para in intellectualRights has children which cannot be shown using the form. Please use Morpho to edit this document";    
                        push(@errorMessages, $error);
                        #if ($DEBUG == 1){ print $error."\n";}
                    } else {
                        #print $tempNode->nodeName().":".$tempNode->textContent();
                        #print "\n";
                        if ($$templateVars{'useConstraints'} eq "") {
                            $$templateVars{'useConstraints'} = $tempNode->textContent();
                        } else {
                            $$templateVars{'useConstraintsOther'} = $tempNode->textContent();
                        }
                    }
                }
            }
        }
    }

    $results = $doc->findnodes('//dataset/distribution/online');
    if ($results->size() > 1) {
        errMoreThanOne("distribution/online");
    } else {
        foreach my $tempNode ($results->get_nodelist){
            $$templateVars{'url'} = findValue($tempNode, "url");
            dontOccur($tempNode, "./connection", "/distribution/online/connection");
            dontOccur($tempNode, "./connectionDefinition", "/distribution/online/connectionDefinition");
        }
    }

    $results = $doc->findnodes('//dataset/distribution/offline');
    if ($results->size() > 1) {
        errMoreThanOne("distribution/online");
    } else {
        foreach my $tempNode ($results->get_nodelist) {
            $$templateVars{'dataMedium'} = findValue($tempNode, "mediumName");
            dontOccur($tempNode, "./mediumDensity", "/distribution/offline/mediumDensity");
            dontOccur($tempNode, "./mediumDensityUnits", "/distribution/offline/mediumDensityUnits");
            dontOccur($tempNode, "./mediumVolume", "/distribution/offline/mediumVolume");
            dontOccur($tempNode, "./mediumFormat", "/distribution/offline/mediumFormat");
            dontOccur($tempNode, "./mediumNote", "/distribution/offline/mediumNote");
        }
    }

    dontOccur($doc, "./inline", "//dataset/distribution/inline");

    $results = $doc->findnodes('//dataset/coverage');
    if ($results->size() > 1) {
        errMoreThanOne("coverage");
    } else {
        foreach $node ($results->get_nodelist) {
            dontOccur($node, "./temporalCoverage/rangeOfDates/beginDate/time|./temporalCoverage/rangeOfDates/beginDate/alternativeTimeScale|./temporalCoverage/rangeOfDates/endDate/time|./temporalCoverage/rangeOfDates/endDate/alternativeTimeScale|./taxonomicCoverage/taxonomicSystem|./taxonomicCoverage/taxonomicClassification/commonName|./taxonomicCoverage/taxonomicClassification/taxonomicClassification|./geographicCoverage/datasetGPolygon|./geographicCoverage/boundingCoordinates/boundingAltitudes", "temporalCoverage/rangeOfDates/beginDate/time, /temporalCoverage/rangeOfDates/beginDate/alternativeTimeScale, /temporalCoverage/rangeOfDates/endDate/time, /temporalCoverage/rangeOfDates/endDate/alternativeTimeScale, /taxonomicCoverage/taxonomicSystem, /taxonomicCoverage/taxonomicClassification/commonName, /taxonomicCoverage/taxonomicClassification/taxonomicClassification, /geographicCoverage/datasetGPolygon, /geographicCoverage/boundingCoordinates/boundingAltitudes");

            $tempResult = $node->findnodes('./temporalCoverage');
            if ($tempResult->size > 1) {
                   errMoreThanOne("temporalCoverage");
            } else {
                foreach $tempNode ($tempResult->get_nodelist) {
                    my $x;
                    my $y;
                    my $z;
                    my $tempdate = findValue($tempNode, "rangeOfDates/beginDate/calendarDate");
                    ($x, $y, $z) = split("-", $tempdate); 
                    $$templateVars{'beginningYear'} = $x;
                    $$templateVars{'beginningMonth'} = $y;
                    $$templateVars{'beginningDay'} = $z;
    
                    $tempdate = findValue($tempNode, "rangeOfDates/endDate/calendarDate");
                    ($x, $y, $z) = split("-", $tempdate);
                    $$templateVars{'endingYear'} = $x;
                    $$templateVars{'endingMonth'} = $y;
                    $$templateVars{'endingDay'} = $z;

                    $tempdate = "";
                    $tempdate = findValue($tempNode, "singleDateTime/calendarDate");
                    if($tempdate ne ""){
                        ($x, $y, $z) = split("-", $tempdate);
                        $$templateVars{'beginningYear'} = $x;
                        $$templateVars{'beginningMonth'} = $y;
                        $$templateVars{'beginningDay'} = $z;
                    }  
		    
		    $$templateVars{'hasTemporal'} = "true";
                }
            }

            $tempResult = $node->findnodes('./geographicCoverage');
            if ($tempResult->size > 1) {
                errMoreThanOne("geographicCoverage");
            } else {
                foreach $tempNode ($tempResult->get_nodelist) {
                    my $geogdesc = findValue($tempNode, "geographicDescription");
                    debug("Registry: geogdesc from xml is: $geogdesc");
                    $$templateVars{'geogdesc'} = $geogdesc;
                    my $coord = findValue($tempNode, "boundingCoordinates/westBoundingCoordinate");
                    if ($coord > 0) {
                        #print "+";
                        $$templateVars{'hemisphLong1'} = "E";
                    } else {
                        #print "-";
                        eval($coord = $coord * -1);
                        $$templateVars{'hemisphLong1'} = "W";
                    }
                    eval($$templateVars{'longDeg1'} = int($coord));
                    eval($coord = ($coord - int($coord))*60);
                    eval($$templateVars{'longMin1'} = int($coord));
                    eval($coord = ($coord - int($coord))*60);
                    eval($$templateVars{'longSec1'} = int($coord));
                    
                    $coord = findValue($tempNode, "boundingCoordinates/southBoundingCoordinate");
                    if ($coord > 0) {
                        #print "+";
                        $$templateVars{'hemisphLat2'} = "N";
                    } else {
                        #print "-";
                        eval($coord = $coord * -1);
                        $$templateVars{'hemisphLat2'} = "S";
                    }
                    eval($$templateVars{'latDeg2'} = int($coord));
                    eval($coord = ($coord - int($coord))*60);
                    eval($$templateVars{'latMin2'} = int($coord));
                    eval($coord = ($coord - int($coord))*60);
                    eval($$templateVars{'latSec2'} = int($coord));
        
                    $coord = findValue($tempNode, "boundingCoordinates/northBoundingCoordinate");
                    if ($coord > 0) {
                        #print "+";
                        $$templateVars{'hemisphLat1'} = "N";
                    } else {
                        #print "-";
                        eval($coord = $coord * -1);
                        $$templateVars{'hemisphLat1'} = "S";
                    }
                    eval($$templateVars{'latDeg1'} = int($coord));
                    eval($coord = ($coord - int($coord))*60);
                    eval($$templateVars{'latMin1'} = int($coord));
                    eval($coord = ($coord - int($coord))*60);
                    eval($$templateVars{'latSec1'} = int($coord));
        
                    $coord = findValue($tempNode, "boundingCoordinates/eastBoundingCoordinate");
                    if ($coord > 0) {
                        #print "+";
                        $$templateVars{'hemisphLong2'} = "E";
                    } else {
                        #print "-";
                        eval($coord = $coord * -1);
                        $$templateVars{'hemisphLong2'} = "W";
                    }
                    eval($$templateVars{'longDeg2'} = int($coord));
                    eval($coord = ($coord - int($coord))*60);
                    eval($$templateVars{'longMin2'} = int($coord));
                    eval($coord = ($coord - int($coord))*60);
                    eval($$templateVars{'longSec2'} = int($coord));

		            $$templateVars{'hasSpatial'} = "true";
                }
            }

            $tempResult = $node->findnodes('./taxonomicCoverage/taxonomicClassification');
            my $taxonIndex = 0;
            foreach $tempNode ($tempResult->get_nodelist) {
                $taxonIndex++;
                my $taxonRankName = findValue($tempNode, "taxonRankName");
                my $taxonRankValue = findValue($tempNode, "taxonRankValue");
                $$templateVars{"taxonRankName".$taxonIndex} = $taxonRankName;
                $$templateVars{"taxonRankValue".$taxonIndex} = $taxonRankValue;
		
		$$templateVars{'hasTaxonomic'} = "true";
            }
            $$templateVars{'taxaCount'} = $taxonIndex;
            my $taxaAuth = findValue($node, "./taxonomicCoverage/generalTaxonomicCoverage");
            $$templateVars{'taxaAuth'} = $taxaAuth;
        }
    }
    dontOccur($doc, "./purpose", "purpose");
    dontOccur($doc, "./maintenance", "maintnance");

    $results = $doc->findnodes('//dataset/contact/individualName');
    if ($results->size() > 1) {
        errMoreThanOne("contact/individualName");
    } else {
        foreach $node ($results->get_nodelist) {
            dontOccur($node, "../positionName|../onlineURL|../userId", 
              "positionName, onlineURL, userId in contact tag");
            dontOccur($node, "./saluation", "saluation in contact tag");                
        
            $tempResult = $node->findnodes('../address|../phone|../electronicmailAddress|../organizationName');
            if ($tempResult->size > 0) {
                $$templateVars{'origNamefirstContact'} = findValue($node, 'givenName');
                $$templateVars{'origNamelastContact'} = findValue($node, 'surName');
    
                my $tempResult2 = $node->findnodes('../address');
                if ($tempResult2->size > 1) {
                    errMoreThanOne("address");
                } else {
                    foreach my $tempNode2 ($tempResult2->get_nodelist) {
                        $$templateVars{'origDeliveryContact'} = findValue($tempNode2, 'deliveryPoint');
                        $$templateVars{'origCityContact'} = findValue($tempNode2, 'city');
                        $$templateVars{'origStateContact'} = findValue($tempNode2, 'administrativeArea');
                        $$templateVars{'origZIPContact'} = findValue($tempNode2, 'postalCode');
                        $$templateVars{'origCountryContact'} = findValue($tempNode2, 'country');
                    }
                }
            
                my $tempResult3 = $node->findnodes('../phone');
                if ($tempResult3->size > 2) {
                    errMoreThanN("phone");
                } else {
                    foreach my $tempNode2 ($tempResult3->get_nodelist) {
                        if ($tempNode2->hasAttributes()) {
                            my @attlist = $tempNode2->attributes();
                            if ($attlist[0]->value eq "Fax") {
                                $$templateVars{'origFAXContact'} = $tempNode2->textContent();
                            } else {
                                $$templateVars{'origPhoneContact'} = $tempNode2->textContent();
                            }
                        } else {
                            $$templateVars{'origPhoneContact'} = $tempNode2->textContent();
                        }
                    }
                }
                $$templateVars{'origEmailContact'} = findValue($node, '../electronicMailAddress');
                $$templateVars{'origNameOrgContact'} = findValue($node, '../organizationName');
            } else {
                $$templateVars{'origNamefirstContact'} = findValue($node, 'givenName');
                $$templateVars{'origNamelastContact'} = findValue($node, 'surName');
                $$templateVars{'origNameOrgContact'} = findValue($node, '../organizationName');
            }
        }
    }
    
    $results = $doc->findnodes(
            '//dataset/methods/methodStep/description/section');
    debug("Registry: Number methods: ".$results->size());
    if ($results->size() > 1) {
        errMoreThanN("methods/methodStep/description/section");    
    } else {

        my @methodPara;
        foreach $node ($results->get_nodelist) {
            my @children = $node->childNodes;
            for (my $i = 0; $i < scalar(@children); $i++) {
                debug("Registry: Method child loop ($i)");
                my $child = $children[$i];
                if ($child->nodeName eq 'title') {
                    my $title = $child->textContent();
                    debug("Registry: Method title ($title)");
                    $$templateVars{'methodTitle'} = $title;
                } elsif ($child->nodeName eq 'para') {
                    my $para = $child->textContent();
                    debug("Registry: Method para ($para)");
                    $methodPara[scalar(@methodPara)] = $para;
                }
            }
	    $$templateVars{'hasMethod'} = "true";
        }
        if (scalar(@methodPara) > 0) {
            $$templateVars{'methodPara'} = \@methodPara;
        }
    }

    $results = $doc->findnodes(
            '//dataset/methods/sampling/studyExtent/description/para');
    if ($results->size() > 1) {
        errMoreThanN("methods/sampling/studyExtent/description/para");    
    } else {
        foreach $node ($results->get_nodelist) {
            my $studyExtentDescription = $node->textContent();
            $$templateVars{'studyExtentDescription'} = $studyExtentDescription;

	    $$templateVars{'hasMethod'} = "true";
        }
    }

    $results = $doc->findnodes(
            '//dataset/methods/sampling/samplingDescription/para');
    if ($results->size() > 1) {
        errMoreThanN("methods/sampling/samplingDescription/para");    
    } else {
        foreach $node ($results->get_nodelist) {
            my $samplingDescription = $node->textContent();
            $$templateVars{'samplingDescription'} = $samplingDescription;

	    $$templateVars{'hasMethod'} = "true";
        }
    }

    dontOccur($doc, "//methodStep/citation", "methodStep/citation");
    dontOccur($doc, "//methodStep/protocol", "methodStep/protocol");
    dontOccur($doc, "//methodStep/instrumentation", "methodStep/instrumentation");
    dontOccur($doc, "//methodStep/software", "methodStep/software");
    dontOccur($doc, "//methodStep/subStep", "methodStep/subStep");
    dontOccur($doc, "//methodStep/dataSource", "methodStep/dataSource");
    dontOccur($doc, "//methods/qualityControl", "methods/qualityControl");

    dontOccur($doc, "//methods/sampling/spatialSamplingUnits", "methods/sampling/spatialSamplingUnits");
    dontOccur($doc, "//methods/sampling/citation", "methods/sampling/citation");
    dontOccur($doc, "./pubPlace", "pubPlace");
    dontOccur($doc, "./project", "project");
    
    ############ Code for checking ACL #####################
    dontOccur($doc, "//dataset/access/deny", "dataset/access/deny");

    $results = $doc->findnodes('//dataset/access/allow');
    if ($results->size() != 3) {
        errMoreThanN("dataset/access/allow");
    } else {
	my $accessError = 0;
        foreach $node ($results->get_nodelist) {
            my @children = $node->childNodes;
	    my $principal = "";
	    my $permission = "";
            for (my $i = 0; $i < scalar(@children); $i++) {
                my $child = $children[$i];
                if ($child->nodeName eq 'principal') {
                    $principal = $child->textContent();
                } elsif ($child->nodeName eq 'permission') {
                    $permission = $child->textContent();
                }
            }
	
	    if ($principal eq 'public' && $permission ne 'read') { $accessError = 1; }
	    if ($principal eq $username && $permission ne 'all') { $accessError = 2; }
	    if ($principal ne 'public' && $principal ne $username && $permission ne 'all') { $accessError = 3; }
	}
 
	if ($accessError != 0) {
	    my $error ="The ACL for this document has been changed outside the registry. Please use Morpho to edit this document";
            push(@errorMessages, $error."\n");
	}     
    }
    ########################################################


    dontOccur($doc, "./dataTable", "dataTable");
    dontOccur($doc, "./spatialRaster", "spatialRaster");
    dontOccur($doc, "./spatialVector", "spatialVector");
    dontOccur($doc, "./storedProcedure", "storedProcedure");
    dontOccur($doc, "./view", "view");
    dontOccur($doc, "./otherEntity", "otherEntity");
    dontOccur($doc, "./references", "references");
    
    dontOccur($doc, "//citation", "citation");
    dontOccur($doc, "//software", "software");
    dontOccur($doc, "//protocol", "protocol");
    dontOccur($doc, "//additionalMetadata", "additionalMetadata");    
}

################################################################################
# 
# Delete the eml file that has been requested for deletion. 
#
################################################################################
sub deleteData {
    my $deleteAll = shift;
    
    # create metacat instance
    my $metacat;
    my $docid = $FORM::docid;
    
    $metacat = Metacat->new();
    if ($metacat) {
        $metacat->set_options( metacatUrl => $metacatUrl );
    } else {
        #die "failed during metacat creation\n";
        push(@errorMessages, "Failed during metacat creation.");
    }

    # Login to metacat
    my $userDN = $FORM::username;
    my $userOrg = $FORM::organization;
    my $userPass = $FORM::password;
    my $dname = "uid=$userDN,o=$userOrg,dc=ecoinformatics,dc=org";
    
    my $errorMessage = "";
    my $response = $metacat->login($dname, $userPass);

    if (! $response) {
    # Could not login
        push(@errorMessages, $metacat->getMessage());
        push(@errorMessages, "Failed during login.\n");

    } else {
    #Able to login - try to delete the file    

    my $parser;
    my @fileArray;
    my $httpMessage;
    my $xmldoc;
    my $doc;
    my $pushDoc;
    my $alreadyInArray;
    my $findType;
        my $node;
    my $response; 
    my $element;

    push (@fileArray, $docid);
    $parser = XML::LibXML->new();

        $httpMessage = $metacat->read($docid);
    $doc = $httpMessage->content();    
    $xmldoc = $parser->parse_string($doc);

    if ($xmldoc eq "") {
        $error ="Error in parsing the eml document";
        push(@errorMessages, $error);
    } else {

        $findType = $xmldoc->findnodes('//dataset/identifier');
        if($findType->size() > 0){
        # This is a eml beta6 document
        # Delete the documents mentioned in triples also
        
        $findType = $xmldoc->findnodes('//dataset/triple');
        if($findType->size() > 0){
            foreach $node ($findType->get_nodelist){
            $pushDoc = findValue($node, 'subject');
            
            # If the file is already in the @fileArray then do not add it 
            $alreadyInArray = 0;
            foreach $element (@fileArray){
                if($element eq $pushDoc){
                $alreadyInArray = 1;
                }
            }
            
            if(!$alreadyInArray){
                # If not already in array then delete the file. 
                push (@fileArray, $pushDoc);
                $response = $metacat->delete($pushDoc);
                
                if (! $response) {
                # Could not delete
                #push(@errorMessages, $response);
                push(@errorMessages, $metacat->getMessage());
                push(@errorMessages, "Failed during deleting $pushDoc. Please check if you are authorized to delete this document.\n");
                }
            }
            }
        }
        }
    }
    
    # Delete the main document. 
    if($deleteAll){
        $response = $metacat->delete($docid);  
        if (! $response) {
        # Could not delete
        #push(@errorMessages, $response);
        push(@errorMessages, $metacat->getMessage());
        push(@errorMessages, "Failed during deleting $docid. Please check if you are authorized to delete this document.\n");
        }
    }
    }
    
    if (scalar(@errorMessages)) {
    # If any errors, print them in the response template 
    $$templateVars{'status'} = 'failure';
    $$templateVars{'errorMessages'} = \@errorMessages;
    $error = 1;
    }
    
    # Process the response template
    if($deleteAll){

    $$templateVars{'function'} = "deleted";
    $$templateVars{'section'} = "Deletion Status";
    $template->process( $responseTemplate, $templateVars);
    }
}


################################################################################
# 
# Do data validation and send the data to confirm data template.
#
################################################################################
sub toConfirmData{
    # Check if any invalid parameters
 
    my $invalidParams;
    if (! $error) {
    $invalidParams = validateParameters(0);
    if (scalar(@$invalidParams)) {
        $$templateVars{'status'} = 'failure';
        $$templateVars{'invalidParams'} = $invalidParams;
        $error = 1;
    }
    }


    $$templateVars{'providerGivenName'} = normalizeCD($FORM::providerGivenName);
    $$templateVars{'providerSurName'} = normalizeCD($FORM::providerSurName);
    if($FORM::site eq "Select your station here."){
        $$templateVars{'site'} = "";
    }else{
        $$templateVars{'site'} = $FORM::site;
    }
    if($FORM::cfg eq "nceas"){
        $$templateVars{'wg'} = \@FORM::wg;
    }
    $$templateVars{'identifier'} = normalizeCD($FORM::identifier);
    $$templateVars{'title'} = normalizeCD($FORM::title);
    $$templateVars{'origNamefirst0'} = normalizeCD($FORM::origNamefirst0);
    $$templateVars{'origNamelast0'} = normalizeCD($FORM::origNamelast0);
    $$templateVars{'origNameOrg'} = normalizeCD($FORM::origNameOrg);
    # $$templateVars{'origRole0'} = $FORM::origRole0;
    $$templateVars{'origDelivery'} = normalizeCD($FORM::origDelivery);
    $$templateVars{'origCity'} = normalizeCD($FORM::origCity);
    if($FORM::origState eq "Select State Here."){
        $$templateVars{'origState'} = "";
    }else{
        $$templateVars{'origState'} = $FORM::origState;
    }
    $$templateVars{'origStateOther'} = normalizeCD($FORM::origStateOther);
    $$templateVars{'origZIP'} = normalizeCD($FORM::origZIP);
    $$templateVars{'origCountry'} = normalizeCD($FORM::origCountry);
    $$templateVars{'origPhone'} = normalizeCD($FORM::origPhone);
    $$templateVars{'origFAX'} = normalizeCD($FORM::origFAX);
    $$templateVars{'origEmail'} = normalizeCD($FORM::origEmail);
    $$templateVars{'useOrigAddress'} = normalizeCD($FORM::useOrigAddress);
    if($FORM::useOrigAddress eq "on"){
        $$templateVars{'origNamefirstContact'} = normalizeCD($FORM::origNamefirst0);
        $$templateVars{'origNamelastContact'} = normalizeCD($FORM::origNamelast0);
        $$templateVars{'origNameOrgContact'} = normalizeCD($FORM::origNameOrg);
        $$templateVars{'origDeliveryContact'} = normalizeCD($FORM::origDelivery); 
        $$templateVars{'origCityContact'} = normalizeCD($FORM::origCity);
        if($FORM::origState eq "Select State Here."){
        $$templateVars{'origStateContact'} = "";
        }else{
        $$templateVars{'origStateContact'} = $FORM::origState;
        }
        $$templateVars{'origStateOtherContact'} = normalizeCD($FORM::origStateOther);
        $$templateVars{'origZIPContact'} = normalizeCD($FORM::origZIP);
        $$templateVars{'origCountryContact'} = normalizeCD($FORM::origCountry);
        $$templateVars{'origPhoneContact'} = normalizeCD($FORM::origPhone);
        $$templateVars{'origFAXContact'} = normalizeCD($FORM::origFAX);
        $$templateVars{'origEmailContact'} = normalizeCD($FORM::origEmail);
    }else{
        $$templateVars{'origNamefirstContact'} = normalizeCD($FORM::origNamefirstContact);
        $$templateVars{'origNamelastContact'} = normalizeCD($FORM::origNamelastContact);
        $$templateVars{'origNameOrgContact'} = normalizeCD($FORM::origNameOrgContact);
        $$templateVars{'origDeliveryContact'} = normalizeCD($FORM::origDeliveryContact); 
        $$templateVars{'origCityContact'} = normalizeCD($FORM::origCityContact);
        if($FORM::origStateContact eq "Select State Here."){
        $$templateVars{'origStateContact'} = "";
        }else{
        $$templateVars{'origStateContact'} = $FORM::origStateContact;
        }
        $$templateVars{'origStateOtherContact'} = normalizeCD($FORM::origStateOtherContact);
        $$templateVars{'origZIPContact'} = normalizeCD($FORM::origZIPContact);
        $$templateVars{'origCountryContact'} = normalizeCD($FORM::origCountryContact);
        $$templateVars{'origPhoneContact'} = normalizeCD($FORM::origPhoneContact);
        $$templateVars{'origFAXContact'} = normalizeCD($FORM::origFAXContact);
        $$templateVars{'origEmailContact'} = normalizeCD($FORM::origEmailContact);    
    }

    $$templateVars{'aoCount'} = $FORM::aoCount;
    foreach my $origName (param()) {
	if ($origName =~ /origNamefirst/) {
	    my $origNameIndex = $origName;
	    $origNameIndex =~ s/origNamefirst//; # get the index of the parameter 0, ..., 10
	    my $origNamelast = "origNamelast".$origNameIndex;
	    my $origRole = "origRole".$origNameIndex;
	    if ( $origNameIndex =~ /[0-9]/  && $origNameIndex > 0){
		if (hasContent(param($origName)) && hasContent(param($origNamelast)) && hasContent(param($origRole))) {
		    debug("Registry processing keyword: $origName = ".param($origName)." $origNamelast = ".param($origNamelast)." $origRole = ".param($origRole));
		    $$templateVars{$origName} = normalizeCD(param($origName));
		    $$templateVars{$origNamelast} = normalizeCD(param($origNamelast));
		    $$templateVars{$origRole} = normalizeCD(param($origRole));
		}
	    }
	}
    }

    $$templateVars{'abstract'} = normalizeCD($FORM::abstract);
    $$templateVars{'keyCount'} = $FORM::keyCount;
    foreach my $kyd (param()) {
	if ($kyd =~ /keyword/) {
	    my $keyIndex = $kyd;
	    $keyIndex =~ s/keyword//; # get the index of the parameter 0, ..., 10
	    my $keyType = "kwType".$keyIndex;
	    my $keyTh = "kwTh".$keyIndex;
	    if ( $keyIndex =~ /[0-9]/ ){
		if (hasContent(param($kyd)) && hasContent(param($keyType)) && hasContent(param($keyTh))) {
		    debug("Registry processing keyword: $kyd = ".param($kyd)." $keyType = ".param($keyType)." $keyTh = ".param($keyTh));
		    $$templateVars{$kyd} = normalizeCD(param($kyd));
		    $$templateVars{$keyType} = param($keyType);
		    $$templateVars{$keyTh} = param($keyTh);
		}
	    }
	}
    }
    $$templateVars{'addComments'} = normalizeCD($FORM::addComments);
    $$templateVars{'useConstraints'} = $FORM::useConstraints;
    $$templateVars{'useConstraintsOther'} = $FORM::useConstraintsOther;
    $$templateVars{'url'} = $FORM::url;
    if($FORM::dataMedium eq "Select type of medium here."){
        $$templateVars{'dataMedium'} = "";
    }else{
        $$templateVars{'dataMedium'} = $FORM::dataMedium;
    }    
    $$templateVars{'dataMediumOther'} = normalizeCD($FORM::dataMediumOther);
    $$templateVars{'beginningYear'} = $FORM::beginningYear;
    $$templateVars{'beginningMonth'} = $FORM::beginningMonth;
    $$templateVars{'beginningDay'} = $FORM::beginningDay;
    $$templateVars{'endingYear'} = $FORM::endingYear;
    $$templateVars{'endingMonth'} = $FORM::endingMonth;
    $$templateVars{'endingDay'} = $FORM::endingDay;
    $$templateVars{'geogdesc'} = normalizeCD($FORM::geogdesc);
    $$templateVars{'useSiteCoord'} = $FORM::useSiteCoord;
    $$templateVars{'latDeg1'} = $FORM::latDeg1;
    $$templateVars{'latMin1'} = $FORM::latMin1;
    $$templateVars{'latSec1'} = $FORM::latSec1;
    $$templateVars{'hemisphLat1'} = $FORM::hemisphLat1;
    $$templateVars{'longDeg1'} = $FORM::longDeg1;
    $$templateVars{'longMin1'} = $FORM::longMin1;
    $$templateVars{'longSec1'} = $FORM::longSec1;
    $$templateVars{'hemisphLong1'} = $FORM::hemisphLong1;
    $$templateVars{'latDeg2'} = $FORM::latDeg2;
    $$templateVars{'latMin2'} = $FORM::latMin2;
    $$templateVars{'latSec2'} = $FORM::latSec2;
    $$templateVars{'hemisphLat2'} = $FORM::hemisphLat2;
    $$templateVars{'longDeg2'} = $FORM::longDeg2;
    $$templateVars{'longMin2'} = $FORM::longMin2;
    $$templateVars{'longSec2'} = $FORM::longSec2;
    $$templateVars{'hemisphLong2'} = $FORM::hemisphLong2;

    $$templateVars{'taxaCount'} = $FORM::taxaCount;
    foreach my $trn (param()) {
        if ($trn =~ /taxonRankName/) {
            my $taxIndex = $trn;
            $taxIndex =~ s/taxonRankName//; # get the index of the parameter 0, ..., 10
            my $trv = "taxonRankValue".$taxIndex;
            if ( $taxIndex =~ /[0-9]/ ){
                if (hasContent(param($trn)) && hasContent(param($trv))) {
                    debug("Registry processing taxon: $trn = ".param($trn)." $trv = ".param($trv));
                    $$templateVars{$trn} = normalizeCD(param($trn));
                    $$templateVars{$trv} = normalizeCD(param($trv));
                }
            }
        }
    }
    $$templateVars{'taxaAuth'} = normalizeCD($FORM::taxaAuth);

    $$templateVars{'methodTitle'} = normalizeCD($FORM::methodTitle);

    my @tempMethodPara;
    for (my $i = 0; $i < scalar(@FORM::methodPara); $i++) {
	$tempMethodPara[$i] = normalizeCD($FORM::methodPara[$i]);
    }
    $$templateVars{'methodPara'} = \@tempMethodPara;
    $$templateVars{'studyExtentDescription'} = normalizeCD($FORM::studyExtentDescription);
    $$templateVars{'samplingDescription'} = normalizeCD($FORM::samplingDescription);
    $$templateVars{'origStateContact'} = $FORM::origState;

    $$templateVars{'showSiteList'} = $FORM::showSiteList;
    $$templateVars{'lsite'} = $FORM::lsite;
    $$templateVars{'usite'} = $FORM::usite;
    $$templateVars{'showWgList'} = $FORM::showWgList;
    $$templateVars{'showOrganization'} = $FORM::showOrganization;
    $$templateVars{'hasKeyword'} = $FORM::hasKeyword;
    $$templateVars{'hasTemporal'} = $FORM::hasTemporal;
    $$templateVars{'hasSpatial'} = $FORM::hasSpatial;
    $$templateVars{'hasTaxonomic'} = $FORM::hasTaxonomic;
    $$templateVars{'hasMethod'} = $FORM::hasMethod;
    $$templateVars{'spatialRequired'} = $FORM::spatialRequired;
    $$templateVars{'temporalRequired'} = $FORM::temporalRequired;

    $$templateVars{'docid'} = $FORM::docid;

    if (! $error) {
	# If no errors, then print out data in confirm Data template

	$$templateVars{'section'} = "Confirm Data";
	$template->process( $confirmDataTemplate, $templateVars);

    } else{    
    # Errors from validation function. print the errors out using the response template
    if (scalar(@errorMessages)) {
        $$templateVars{'status'} = 'failure';
        $$templateVars{'errorMessages'} = \@errorMessages;
        $error = 1;
    }
        # Create our HTML response and send it back
    $$templateVars{'function'} = "submitted";
    $$templateVars{'section'} = "Submission Status";
    $template->process( $responseTemplate, $templateVars);
    }
}


################################################################################
# 
# From confirm Data template - user wants to make some changes.
#
################################################################################
sub confirmDataToReEntryData{
    my @sortedSites;
    foreach my $site (sort @sitelist) {
        push(@sortedSites, $site);
    }

    $$templateVars{'siteList'} = \@sortedSites;
    $$templateVars{'section'} = "Re-Entry Form";
    copyFormToTemplateVars();
    $$templateVars{'docid'} = $FORM::docid;

    $$templateVars{'form'} = 're_entry';
    $template->process( $entryFormTemplate, $templateVars);
}


################################################################################
# 
# Copy form data to templateVars.....
#
################################################################################
sub copyFormToTemplateVars{
    $$templateVars{'providerGivenName'} = $FORM::providerGivenName;
    $$templateVars{'providerSurName'} = $FORM::providerSurName;
    $$templateVars{'site'} = $FORM::site;
    if ($FORM::cfg eq "nceas") {
        my $projects = getProjectList();
        $$templateVars{'projects'} = $projects;
        $$templateVars{'wg'} = \@FORM::wg;
    }
    $$templateVars{'identifier'} = $FORM::identifier;
    $$templateVars{'title'} = $FORM::title;
    $$templateVars{'origNamefirst0'} = $FORM::origNamefirst0;
    $$templateVars{'origNamelast0'} = $FORM::origNamelast0;
    $$templateVars{'origNameOrg'} = $FORM::origNameOrg;
 #   $$templateVars{'origRole0'} = $FORM::origRole0;
    $$templateVars{'origDelivery'} = $FORM::origDelivery;
    $$templateVars{'origCity'} = $FORM::origCity;
    $$templateVars{'origState'} = $FORM::origState;
    $$templateVars{'origStateOther'} = $FORM::origStateOther;
    $$templateVars{'origZIP'} = $FORM::origZIP;
    $$templateVars{'origCountry'} = $FORM::origCountry;
    $$templateVars{'origPhone'} = $FORM::origPhone;
    $$templateVars{'origFAX'} = $FORM::origFAX;
    $$templateVars{'origEmail'} = $FORM::origEmail;
    if ($FORM::useSiteCoord ne "") {
        $$templateVars{'useOrigAddress'} = "CHECKED";
    }else{
        $$templateVars{'useOrigAddress'} = $FORM::useOrigAddress;
    }
    $$templateVars{'origNamefirstContact'} = $FORM::origNamefirstContact;
    $$templateVars{'origNamelastContact'} = $FORM::origNamelastContact;
    $$templateVars{'origNameOrgContact'} = $FORM::origNameOrgContact;
    $$templateVars{'origDeliveryContact'} = $FORM::origDeliveryContact; 
    $$templateVars{'origCityContact'} = $FORM::origCityContact;
    $$templateVars{'origStateContact'} = $FORM::origStateContact;
    $$templateVars{'origStateOtherContact'} = $FORM::origStateOtherContact;
    $$templateVars{'origZIPContact'} = $FORM::origZIPContact;
    $$templateVars{'origCountryContact'} = $FORM::origCountryContact;
    $$templateVars{'origPhoneContact'} = $FORM::origPhoneContact;
    $$templateVars{'origFAXContact'} = $FORM::origFAXContact;
    $$templateVars{'origEmailContact'} = $FORM::origEmailContact;    
    
    $$templateVars{'aoCount'} = $FORM::aoCount;
    foreach my $origName (param()) {
	if ($origName =~ /origNamefirst/) {
	    my $origNameIndex = $origName;
	    $origNameIndex =~ s/origNamefirst//; # get the index of the parameter 0, ..., 10
	    my $origNamelast = "origNamelast".$origNameIndex;
	    my $origRole = "origRole".$origNameIndex;
	    if ( $origNameIndex =~ /[0-9]/  && $origNameIndex > 0){
		if (hasContent(param($origName)) && hasContent(param($origNamelast)) && hasContent(param($origRole))) {
		    debug("Registry processing keyword: $origName = ".param($origName)." $origNamelast = ".param($origNamelast)." $origRole = ".param($origRole));
		    $$templateVars{$origName} = normalizeCD(param($origName));
		    $$templateVars{$origNamelast} = normalizeCD(param($origNamelast));
		    $$templateVars{$origRole} = normalizeCD(param($origRole));
		}
	    }
	}
    }

    $$templateVars{'abstract'} = $FORM::abstract;
    $$templateVars{'keyCount'} = $FORM::keyCount;
    foreach my $kyd (param()) {
	if ($kyd =~ /keyword/) {
	    my $keyIndex = $kyd;
	    $keyIndex =~ s/keyword//; # get the index of the parameter 0, ..., 10
	    my $keyType = "kwType".$keyIndex;
	    my $keyTh = "kwTh".$keyIndex;
	    if ( $keyIndex =~ /[0-9]/ ){
		if (hasContent(param($kyd)) && hasContent(param($keyType)) && hasContent(param($keyTh))) {
		    debug("Registry processing keyword: $kyd = ".param($kyd)." $keyType = ".param($keyType)." $keyTh = ".param($keyTh));
		    $$templateVars{$kyd} = param($kyd);
		    $$templateVars{$keyType} = param($keyType);
		    $$templateVars{$keyTh} = param($keyTh);
		}
	    }
	}
    }
    $$templateVars{'addComments'} = $FORM::addComments;
    $$templateVars{'useConstraints'} = $FORM::useConstraints;
    $$templateVars{'useConstraintsOther'} = $FORM::useConstraintsOther;
    $$templateVars{'url'} = $FORM::url;
    $$templateVars{'dataMedium'} = $FORM::dataMedium;
    $$templateVars{'dataMediumOther'} = $FORM::dataMediumOther;
    $$templateVars{'beginningYear'} = $FORM::beginningYear;
    $$templateVars{'beginningMonth'} = $FORM::beginningMonth;
    $$templateVars{'beginningDay'} = $FORM::beginningDay;
    $$templateVars{'endingYear'} = $FORM::endingYear;
    $$templateVars{'endingMonth'} = $FORM::endingMonth;
    $$templateVars{'endingDay'} = $FORM::endingDay;
    $$templateVars{'geogdesc'} = $FORM::geogdesc;
    if($FORM::useSiteCoord ne ""){
    $$templateVars{'useSiteCoord'} = "CHECKED";
    }else{
    $$templateVars{'useSiteCoord'} = "";
    }
    $$templateVars{'latDeg1'} = $FORM::latDeg1;
    $$templateVars{'latMin1'} = $FORM::latMin1;
    $$templateVars{'latSec1'} = $FORM::latSec1;
    $$templateVars{'hemisphLat1'} = $FORM::hemisphLat1;
    $$templateVars{'longDeg1'} = $FORM::longDeg1;
    $$templateVars{'longMin1'} = $FORM::longMin1;
    $$templateVars{'longSec1'} = $FORM::longSec1;
    $$templateVars{'hemisphLong1'} = $FORM::hemisphLong1;
    $$templateVars{'latDeg2'} = $FORM::latDeg2;
    $$templateVars{'latMin2'} = $FORM::latMin2;
    $$templateVars{'latSec2'} = $FORM::latSec2;
    $$templateVars{'hemisphLat2'} = $FORM::hemisphLat2;
    $$templateVars{'longDeg2'} = $FORM::longDeg2;
    $$templateVars{'longMin2'} = $FORM::longMin2;
    $$templateVars{'longSec2'} = $FORM::longSec2;
    $$templateVars{'hemisphLong2'} = $FORM::hemisphLong2;
    $$templateVars{'taxaCount'} = $FORM::taxaCount;
    foreach my $trn (param()) {
        if ($trn =~ /taxonRankName/) {
            my $taxIndex = $trn;
            $taxIndex =~ s/taxonRankName//; # get the index of the parameter 0, ..., 10
            my $trv = "taxonRankValue".$taxIndex;
            if ( $taxIndex =~ /[0-9]/ ){
                if (hasContent(param($trn)) && hasContent(param($trv))) {
                    debug("Registry processing taxon: $trn = ".param($trn)." $trv = ".param($trv));
                    $$templateVars{$trn} = param($trn);
                    $$templateVars{$trv} = param($trv);
                }
            }
        }
    }
    $$templateVars{'taxaAuth'} = $FORM::taxaAuth;
    $$templateVars{'methodTitle'} = $FORM::methodTitle;
    $$templateVars{'methodPara'} = \@FORM::methodPara;
    $$templateVars{'studyExtentDescription'} = $FORM::studyExtentDescription;
    $$templateVars{'samplingDescription'} = $FORM::samplingDescription;
    
    $$templateVars{'showSiteList'} = $FORM::showSiteList;
    $$templateVars{'lsite'} = $FORM::lsite;
    $$templateVars{'usite'} = $FORM::usite;
    $$templateVars{'showWgList'} = $FORM::showWgList;
    $$templateVars{'showOrganization'} = $FORM::showOrganization;
    $$templateVars{'hasKeyword'} = $FORM::hasKeyword;
    $$templateVars{'hasTemporal'} = $FORM::hasTemporal;
    $$templateVars{'hasSpatial'} = $FORM::hasSpatial;
    $$templateVars{'hasTaxonomic'} = $FORM::hasTaxonomic;
    $$templateVars{'hasMethod'} = $FORM::hasMethod;
    $$templateVars{'spatialRequired'} = $FORM::spatialRequired;
    $$templateVars{'temporalRequired'} = $FORM::temporalRequired;
}

################################################################################
# 
# check if there is multiple occurence of the given tag and find its value.
#
################################################################################

sub findValue {
    my $node = shift;
    my $value = shift;
    my $result;
    my $tempNode;

    $result = $node->findnodes("./$value");
    if ($result->size > 1) {
        errMoreThanOne("$value");
    } else {
        foreach $tempNode ($result->get_nodelist){
            #print $tempNode->nodeName().":".$tempNode->textContent();
            #print "\n";
            return $tempNode->textContent();
        }
    }
}


################################################################################
# 
# check if given tags has any children. if not return the value
#
################################################################################
sub findValueNoChild {
    my $node = shift;
    my $value = shift;
    my $tempNode;
    my $childNodes;
    my $result;
    my $error;

    $result = $node->findnodes("./$value");
    if($result->size > 1){
       errMoreThanOne("$value");
    } else {
        foreach $tempNode ($result->get_nodelist) {
            $childNodes = $tempNode->childNodes;
            if ($childNodes->size() > 1) {
                $error ="The tag $value has children which cannot be shown using the form. Please use Morpho to edit this document";    
                push(@errorMessages, $error);
                #if ($DEBUG == 1){ print $error."\n";}
            } else {
                #print $tempNode->nodeName().":".$tempNode->textContent();
                #print "\n";
                return $tempNode->textContent();
            }
        }
    }
}


################################################################################
# 
# check if given tags are children of given node.
#
################################################################################
sub dontOccur {
    my $node = shift;
    my $value = shift;
    my $errVal = shift;

    my $result = $node->findnodes("$value");
    if($result->size > 0){
        $error ="One of the following tags found: $errVal. Please use Morpho to edit this document";
        push(@errorMessages, $error."\n");
        #if ($DEBUG == 1){ print $error;}
    } 
}


################################################################################
# 
# print out error for more than one occurence of a given tag
#
################################################################################
sub errMoreThanOne {
    my $value = shift;
    my $error ="More than one occurence of the tag $value found. Please use Morpho to edit this document";
    push(@errorMessages, $error."\n");
    # if ($DEBUG == 1){ print $error;}
}


################################################################################
# 
# print out error for more than given number of occurences of a given tag
#
################################################################################
sub errMoreThanN {
    my $value = shift;
    my $error ="More occurences of the tag $value found than that can be shown in the form. Please use Morpho to edit this document";
    push(@errorMessages, $error);
    #if ($DEBUG == 1){ print $error."\n";}
}


################################################################################
# 
# convert coord to degrees, minutes and seconds form. 
#
################################################################################
#sub convertCoord {
#    my $wx = shift;
#    print $deg." ".$min." ".$sec;
#    print "\n";
#}


################################################################################
# 
# print debugging messages to stderr
#
################################################################################
sub debug {
    my $msg = shift;
    
    if ($debug) {
        print STDERR "$msg\n";
    }
}

################################################################################
# 
# get the list of projects
#
################################################################################
sub getProjectList {
    
    use NCEAS::AdminDB;
    my $admindb = NCEAS::AdminDB->new();
    $admindb->connect($nceas_db, $nceas_db_user, $nceas_db_password);
    my $projects = $admindb->getProjects();
    #my $projects = getTestProjectList();
    return $projects;
}

################################################################################
# 
# get a test list of projects for use only in testing where the NCEAS
# admin db is not available.
#
################################################################################
sub getTestProjectList {
    # This block is for testing only!  Remove for production use
    my @row1;
    $row1[0] = 6000; $row1[1] = 'Andelman'; $row1[2] = 'Sandy'; $row1[3] = 'The very long and windy path to an apparent ecological conclusion: statistics lie';
    my @row2; 
    $row2[0] = 7000; $row2[1] = 'Bascompte'; $row2[2] = 'Jordi'; $row2[3] = 'Postdoctoral Fellow';
    my @row3; 
    $row3[0] = 7001; $row3[1] = 'Hackett'; $row3[2] = 'Edward'; $row3[3] = 'Sociology rules the world';
    my @row4; 
    $row4[0] = 7002; $row4[1] = 'Jones'; $row4[2] = 'Matthew'; $row4[3] = 'Informatics rules the world';
    my @row5; 
    $row5[0] = 7003; $row5[1] = 'Schildhauer'; $row5[2] = 'Mark'; $row5[3] = 'Excel rocks my world, assuming a, b, and c';
    my @row6; 
    $row6[0] = 7004; $row6[1] = 'Rogers'; $row6[2] = 'Bill'; $row6[3] = 'Graduate Intern';
    my @row7; 
    $row7[0] = 7005; $row7[1] = 'Zedfried'; $row7[2] = 'Karl'; $row7[3] = 'A multivariate analysis of thing that go bump in the night';
    my @projects;
    $projects[0] = \@row1;
    $projects[1] = \@row2;
    $projects[2] = \@row3;
    $projects[3] = \@row4;
    $projects[4] = \@row5;
    $projects[5] = \@row6;
    $projects[6] = \@row7;
    return \@projects;
}
